Type: Package
Package: ipeaplot
Title: Add Ipea Editorial Standards to 'ggplot2' Graphics
Version: 0.5.2
Authors@R: c(
    person("Pedro", "Ferreira", role=c("aut","cre"), email="pedro.ferreira2@ipea.gov.br"),
    person("Pedro","Jorge", role=c("aut"), email="pedro.alves@ipea.gov.br"),
    person("Daniel","Lima", role="aut", email="daniel.viegas@ipea.gov.br"),
    person("Gustavo","Coelho", role="aut", email="gustavo.coelho@ipea.gov.br"),
    person("Rafael H. M.", "Pereira", role = "aut", email="rafa.pereira.br@gmail.com"),
    person("Lucas","Mation", role="aut", email="lucas.mation@ipea.gov.br"),
    person("Fabio","Vaz", role="ctb", email="fabio.vaz@ipea.gov.br"),
    person("Ipea - Institue for Applied Economic Research", role = c("cph", "fnd")))
Maintainer: Pedro Ferreira <pedro.ferreira2@ipea.gov.br>
Description: Convenient functions to create 'ggplot2' graphics following the 
             editorial guidelines of the  Institute for Applied Economic 
             Research (Ipea).
License: MIT + file LICENSE
URL: https://github.com/ipeadata-lab/ipeaplot
BugReports: https://github.com/ipeadata-lab/ipeaplot/issues
Depends: R (>= 3.3.2)
Imports: checkmate, ggplot2, grDevices, paletteer, scales, rlang,
        ggthemes, svglite, ragg
Suggests: abjData, data.table, dplyr, forcats, geobr, knitr, patchwork,
        purrr, reshape, rmarkdown, sf, sysfonts, testthat, tidyr
VignetteBuilder: knitr
Encoding: UTF-8
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-05-20 12:45:01 UTC; caval
Author: Pedro Ferreira [aut, cre],
  Pedro Jorge [aut],
  Daniel Lima [aut],
  Gustavo Coelho [aut],
  Rafael H. M. Pereira [aut],
  Lucas Mation [aut],
  Fabio Vaz [ctb],
  Ipea - Institue for Applied Economic Research [cph, fnd]
Repository: CRAN
Date/Publication: 2026-05-20 13:10:02 UTC
Built: R 4.6.0; ; 2026-05-25 03:59:36 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(ggplot_add,scale_auto_ipea)
S3method(ggplot_add,scale_ipea_color)
S3method(ggplot_add,scale_ipea_fill)
export(ipea_pal)
export(save_ipeaplot)
export(scale_color_ipea)
export(scale_fill_ipea)
export(theme_ipea)
import(ggplot2)
import(ggthemes)
import(ragg)
import(rlang)
importFrom(grDevices,dev.off)
importFrom(grDevices,postscript)

[![CRAN/METACRAN Version](https://www.r-pkg.org/badges/version/ipeaplot)](https://CRAN.R-project.org/package=ipeaplot)
[![CRAN/METACRAN Total downloads](https://cranlogs.r-pkg.org/badges/grand-total/ipeaplot?color=green)](https://CRAN.R-project.org/package=ipeaplot)
[![rcmdcheck](https://github.com/ipeadata-lab/ipeaplot/workflows/R-CMD-check/badge.svg)](https://github.com/ipeadata-lab/ipeaplot/actions)
[![Lifecycle: stable](https://img.shields.io/badge/lifecycle-stable-brightgreen.svg)](https://lifecycle.r-lib.org/articles/stages.html)

# ipeaplot: criando graficos no padrão editorial do Ipea 

<img align="right" src="man/figures/ipeaplot.png?raw=true" alt="logo" width="140"> 

O **ipeaplot** é um pacote em R desenvolvido para facilitar a padronização de gráficos e figuras seguindo a política editorial do Instituto de Pesquisa Econômica Aplicada (Ipea). O pacote traz algumas funções que facilitam o ajuste de formatação e de cores de gráficos criados com o pacote `ggplot2`, garantindo a conformidade com boa parte das diretrizes das publicações do Instituto.

# Instalação

Você pode instalar o **ipeaplot** direto CRAN:

```
# CRAN
install.packages("ipeaplot")
```

A versão em desenvolvimento está disponível no GitHub:

```
# Versão em desenvolvimento (0.5.0.9)
utils::remove.packages("ipeaplot")
remotes::install_github("ipeadata-lab/ipeaplot")
```
# Visão geral do pacote

O pacote **ipeaplot** foi desenhado para ser usado em conjunto com o pacote [ggplot2](https://ggplot2.tidyverse.org/), a biblioteca mais popular de visualização de dados em R. Atualmente, o **ipeaplot** inclui quatro funções principais:

1. `theme_ipea()` - para formatação de elementos estilísticos da figura (eixos, fontes, linhas, grid, etc.);
2. `scale_color_ipea()` e `scale_fill_ipea()`- selecionam paleta de cores dentro de um conjunto de opções utilizadas pelo Ipea;
3. `save_ipeaplot()` - uma função unificada para salvar gráficos em múltiplos formatos (vetoriais como `.eps` e `.pdf`, e raster como `.png` e `.jpg`), com padrões que facilitam o uso.


# Demonstração do pacote

```
# Carregue os pacotes
library(ipeaplot)
library(ggplot2)
```
No painel abaixo, à direita, temos a representação de um gráfico no formato padrão do *ggplot2*. Já à esquerda, destacamos o resultado obtido após a aplicação de linhas de código com as funções theme_ipea() e scale_color_ipea().

```
# Figura ggplot original
fig_raw <- ggplot(data = mtcars, aes(x = hp , y = mpg, color = cyl)) +
              geom_point() +
              labs(y='Consumo de Combustível',
                   x ='Potência (Número de cavalos)',
                   color='Cilindradas')

# Adicionando os padrões do Ipea
fig_ipea <- fig_raw +
      scale_color_ipea() +
      theme_ipea()

# Salvando o gráfico em múltiplos formatos com a nova função
save_ipeaplot(fig_ipea, file.name = "meu_grafico_ipea", format = c("png", "pdf"))
```
<img src="man/figures/figura_read_me.png" width="70%" style="display: block; margin: auto;" />

Mais detalhes sobre a utilização do **ipeaplot** nas vinhetas:

- [Introdução ao ipeaplot](https://ipeadata-lab.github.io/ipeaplot/articles/intro_ipeaplot.html)

- [Cookbook ipeaplot](https://ipeadata-lab.github.io/ipeaplot/articles/cookbook.html)

# Fontes

Por limitações de licenciamento, o **ipeaplot** utiliza, em títulos e rótulos das figuras, a fonte sem serifa padrão do sistema operacional do usuário e não a tipografia recomendada pelo Editorial do Ipea (*Frutiger*). Se as figuras forem salvas em formatos vetoriais editáveis (como `pdf` ou `eps`), usando a função `save_ipeaplot()`, a equipe editorial poderá alterar a fonte no momento da edição de uma publicação.

# Equipe 

<img align="right" src="https://github.com/ipeadata-lab/ipeaplot/blob/master/man/figures/ipea.png?raw=true" alt="ipea" width="300">

O pacote **ipeaplot** é desenvolvido pela equipe da Coordenação-Geral de Ciência de Dados e Tecnologia da Informação (CGDTI) do Instituto de Pesquisa Econômica Aplicada (Ipea).

# Outros pacotes semelhantes
- UNHCR\'s [unhcrthemes](https://github.com/unhcr-dataviz/unhcrthemes)
- BBC\'s [bbplot](https://github.com/bbc/bbplot)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = identical(tolower(Sys.getenv("NOT_CRAN")), "true"),
  out.width = "100%"
)

## ----eval=FALSE, message=FALSE, warning=FALSE, results='hide'-----------------
# # Leitura de bibliotecas
# library(ipeaplot)
# library(ggplot2)
# library(dplyr)
# library(data.table)
# library(abjData)
# library(reshape)
# library(knitr)
# 

## ----eval=FALSE, message=FALSE, warning=FALSE---------------------------------
# # Leitura dos dados
# ## Gráfico 1
# Ano <- factor(c(2000,2005,2010,2015,2020,2025,2030,2035,2040,2045,2050))
# Populacao <- c(6.1,6.5,6.9,7.3,7.7,8.0,8.3,8.6,8.9,9.1,9.3)
# labels <- as.character(Populacao)
# labels <- gsub("." , "," , labels, fixed=T)
# base_graf1 <- data.frame(Ano,Populacao,labels)
# 
# ## Gráfico 2
# Ano <- factor(c(1950,	1951,	1952,	1953,	1954,	1955,	1956,	1957,	1958,	
#                 1959,	1960,	1961,	1962,	1963,	1964,	1965,	1966,	1967,
#                 1968,	1969,	1970,	1971,	1972,	1973,	1974,	1975,	1976,
#                 1977,	1978,	1979,	1980,	1981,	1982,	1983,	1984,	1985,
#                 1986,	1987,	1988,	1989,	1990,	1991,	1992,	1993,	1994,
#                 1995,	1996,	1997,	1998,	1999,	2000,	2001,	2002,	2003,
#                 2004,	2005,	2006,	2007,	2008,	2009,	2010,	2011,	2012,
#                 2013,	2014))
# Populacao<- c( 1.850116458,	1.791417471,	1.760491225,	1.751060624,
#                1.757374206,	1.773968217,	1.795950884,	1.81926587,
#                1.840930397,	1.860403495,	1.879485845,	1.90178203,
#                1.931384556,	1.96968284 ,	2.011263472,  2.052385316,
#                2.083875031,	2.098137208,	2.091095193,	2.06831928,
#                2.043373483,	2.018775478,	1.985792138,	1.944170268,
#                1.898407101,	1.850171278,	1.807798778,	1.779963926,
#                1.770930519,	1.77532254 ,	1.778741317,	1.77934085,	
#                1.787498884,	1.803957627,	1.823106379,	1.844715108,
#                1.856490979,	1.844087147,	1.801286156,	1.7376904,
#                1.669804195,	1.607655387,	1.548374095,	1.4949701,
#                1.447065626,	1.399607643,  1.354384982,	1.318725392,
#                1.294773864,	1.279786607,	1.268049775,	1.257045807,
#                1.248951736,	1.243210965,	1.239058923,	1.236026043,
#                1.23309632,	1.229128787,	1.223387196,	1.215887987,
#                1.207869122,	1.198749205,	1.186540058,	1.170622969,
#                1.151783904)
# labels <- as.character(Populacao)
# labels <- gsub("." , "," , labels, fixed=T)
# base_graf2<- data.frame(Ano,Populacao, labels)
# 
# ## Gráfico 3
# # Valores Hipoteticos
# Pais <- factor(c("Ìndia","Japão","Alemanha","Reino Unido","França",
#                  "Arábia Saudita","Rússia","China","Estados Unidos"))
# Gasto <- c(40,50,50,60,70,40,90,207,640)
# base_graf3<- data.frame(Pais,Gasto)
# base_graf3$pos <- ifelse(base_graf3$Pais == 'Estados Unidos',2,1)
# 
# library(dplyr)
# base_graf3 <- base_graf3 %>%
#   group_by(pos) %>%
#   mutate(total = sum(Gasto))
# 
# ## Gráfico 4
# Ano <- seq(2009,2020,1)
# Exabytes <- c(130,360,750,1400,3100,6000,10000,14000,19000,25000,31500,40000)
# 
# base_graf4 <- data.frame(Ano,Exabytes)
# 
# ## Gráfico 5
# 
# Ano <- seq(2004,2013,1)
# Mundo <- c(4.2,3.8,4.1,4,1.5,-2,4,2.9,2.4,2.1)
# UE <- c(2.6,2.2,3.4,3.2,0.6,-4.6,2.1,1.7,-0.4,0)
# 
# base_graf5 <- data.frame(Ano,Mundo,UE)
# base_graf5<- melt(base_graf5, id=c("Ano"))
# 
# ## Gráfico 6
# pais <- c("Áustria","Bélgica","Bulgária","Croácia","Chipre","República Tcheca",
#           "Dinamarca","Estônia","Finlândia","França","Alemanha","Grécia",
#           "Hungria","Irlanda","Itália","Letônia","Lituânia","Luxemburgo",
#           "Malta","Holanda","Polônia","Portugal","România",
#           "Eslováquia","Eslovênia","Espanha","Suécia","Reino Unido")
# 
# petroleo <- c(90,100,97,72,104,96,-37,60,90,97,96,100,
#               81,102,89,104,90,98,98,97,96,98,52,89,107,96,95,38)
# 
# base_graf6 <- data.frame(pais,petroleo)
# 
# ## Gráfico 7
# pais <- c("África do Sul","Brasil","Federação Russa",
#           "China","Índia","OCDE")
# 
# final <- c(0.695,0.55,0.43,0.413,0.38,0.315)
# 
# inicio <- c(0.67,0.61,0.4,0.33,0.32,0.297)
# 
# base_graf7 <- data.frame(pais,final,inicio)
# 
# ## Gráfico 8
# # link <- 'https://github.com/ipeadata-lab/ipeaplot/files'
# # base_graf8 <- data.table::fread(paste0(link,'/12239277/dados_grafico9.csv'),
# #                                 dec = ",", encoding = 'Latin-1')
# 
# base_graf8 <- data.table(pais = c("Nigéria", "Vietnã", "Índia", "Indonésia",
#                          "Malásia", "China", "Arábia Saudita", "África do Sul", "México",
#                          "Turquia", "Brasil", "Argentina", "Austrália", "Rússia", "Polônia",
#                          "Canadá", "USA", "Fraça", "Reino Unido", "Espanha", "Coreia do Sul",
#                          "Itália", "Japão", "Alemanha"),
#                 `crescimento medio do PIB per capita_perct` = c(2,0.5, 0.8, 0.5, 1, 0.1, 1.1, 0.4, 0.5, 0.5, 0.3, 0.5, 0.7, -0.2,
#                                                                 -0.1, 0.5, 0.5, 0.3, 0.32, 0.3, 0.12, 0.1, -0.1, 0),
#                 `crescimento populacional medio_perct` = c(3.9,5.3, 4.6, 4.2, 3.65, 4.2, 3.1, 3.62, 3.46, 3.3, 3.4, 3.1, 2.5,
#                                                            3.37, 3.1, 2.2, 2.2, 2.2, 2.18, 2.1, 2.28, 1.9, 2.1, 1.8),
#                 `crescimento do PIB (ppc)_perct` = c(5.9,5.8, 5.4, 4.7, 4.65, 4.3, 4.2, 4.02, 3.96, 3.8, 3.7, 3.6, 3.2,
#                                                      3.17, 3, 2.7, 2.7, 2.5, 2.5, 2.4, 2.4, 2, 2, 1.8))

## ----echo=FALSE, message=FALSE, warning=FALSE, fig.align="center", out.width = "100%"----
#  knitr::include_graphics(paste0(getwd(),"/prints_original/grafico1.png"))
# #![graph](prints_original/grafico1.png){fig-align="center", width='80%'}

## ----eval=FALSE, message=FALSE, warning=FALSE---------------------------------
# graf1 <- ggplot(data=base_graf1, aes(x=as.numeric(as.character(Ano)), y=Populacao)) +
#   geom_bar(stat="identity", fill = '#015f96', width=2) +
#   geom_rug(aes(x = as.numeric(as.character(Ano)) - 2.5),
#            outside = TRUE, sides = "b", length = unit(2, "mm"), linewidth = 0.25) +
#   coord_cartesian(expand = TRUE, clip = "off") +
#   labs(fill = "",
#        title="GRÁFICO 1",
#        subtitle="Evolução do crescimento da população mundial 2000-2050\n(Em bilhões)",
#        caption = 'Fonte: United Nations (2012a).\nTradução dos autores.') +
#   theme_ipea(x_breaks = 11,include_ticks = F,
#              include_x_text_title = F,include_y_text_title = F) +
#   geom_text(aes(label = gsub("\\.", ",",round(Populacao,1))),
#             hjust = 0.5, vjust = -0.5)

## ----eval=FALSE, message=FALSE, warning=FALSE, results='hide'-----------------
# png(paste0(getwd(),"/prints_ipeaplot/grafico1.png"),  width=724, height=546)
# print(graf1)
# dev.off()
# 

## ----echo=FALSE, message=FALSE, warning=FALSE, fig.align="center", out.width = "100%"----
#  knitr::include_graphics(paste0(getwd(),"/prints_ipeaplot/grafico1.png"))

## ----echo=FALSE, message=FALSE, warning=FALSE, fig.align="center", fig.width=6, fig.height=6----
# knitr::include_graphics(paste0(getwd(),"/prints_original/grafico2.png"))

## ----eval=FALSE, message=FALSE, warning=FALSE---------------------------------
# base_graf2$Ano <- as.numeric(as.character(base_graf2$Ano))
# ref <- (subset(base_graf2,substr(Ano,4,4) %in% c('5','0')))
# 
# graf2 <- ggplot(base_graf2, aes(x=Ano, y=Populacao, group = 1))+
#   geom_line(color = '#015f96') +
#   labs(fill = "",
#        title="GRÁFICO 2",
#        subtitle="Taxa de crescimento da população mundial (Em %)",
#        caption = 'Elaboração dos autores.') +
#   theme_ipea(x_breaks = 14, x_text_angle = 90, include_ticks = F,expand_y_limit = T,
#              include_x_text_title = F,include_y_text_title = F) +
#   geom_rug(data = ref, aes(x = Ano+2),inherit.aes = FALSE,
#            outside = TRUE, sides = "b", length = unit(2, "mm"), linewidth = 0.25) +
#   coord_cartesian(clip = "off")

## ----eval=FALSE, message=FALSE, warning=FALSE, results='hide'-----------------
# png(paste0(getwd(),"/prints_ipeaplot/grafico2.png"),  width=724, height=546)
# print(graf2)
# dev.off()
# 

## ----echo=FALSE, message=FALSE, warning=FALSE, fig.align="center", out.width = "100%"----
#  knitr::include_graphics(paste0(getwd(),"/prints_ipeaplot/grafico2.png"))
# 

## ----echo=FALSE, message=FALSE, warning=FALSE, fig.align="center", fig.width=6, fig.height=6----
# knitr::include_graphics(paste0(getwd(),"/prints_original/grafico3.png"))

## ----eval=FALSE, message=FALSE, warning=FALSE---------------------------------
# temp <- base_graf3 %>% distinct(pos, .keep_all = T)
# 
# 
# graf3 <- ggplot(base_graf3, aes(x= as.character(pos), y=Gasto, fill=Pais,
#                                 label = Pais))+
#   geom_bar(width = 0.5, stat = "identity", color= NA) +
#   labs(y="Gastos com defesa",
#        fill = "",
#        title="GRÁFICO 3",
#        subtitle="Gastos com defesa (Em bilhões de dólares)",
#        caption = 'Fonte: Peter G. Peterson Foundation (2013).\nTradução dos autores.') +
#   scale_fill_ipea() +
#   geom_text( position = position_stack(vjust = 0.5), color = 'white') +
#   geom_text(data = temp, aes(y = total ,label = paste0("U$ ",total)), vjust = -.5) +
#   theme_ipea(legend.position="none",
#              include_x_text_title = F) +
#   theme(axis.ticks.x = ggplot2::element_blank(),
#         axis.text.x  = ggplot2::element_blank())

## ----eval=FALSE, message=FALSE, warning=FALSE, results='hide'-----------------
# png(paste0(getwd(),"/prints_ipeaplot/grafico3.png"),  width=724, height=546)
# print(graf3)
# dev.off()
# 

## ----echo=FALSE, message=FALSE, warning=FALSE, fig.align="center", out.width = "100%"----
#  knitr::include_graphics(paste0(getwd(),"/prints_ipeaplot/grafico3.png"))

## ----echo=FALSE, message=FALSE, warning=FALSE, fig.align="center", fig.width=6, fig.height=6----
# knitr::include_graphics(paste0(getwd(),"/prints_original/grafico4.png"))

## ----eval=FALSE, message=FALSE------------------------------------------------
# graf4 <- ggplot(base_graf4, aes(x=Ano)) +
#   geom_ribbon(aes(ymin=0, ymax=Exabytes), fill = "#015f96")+
#   labs(fill = "",
#        title="GRÁFICO 4",
#        subtitle="Evolução do volume de informação digital até 2020",
#        caption = 'Fonte: IDC’s Digital Universe Study (Gantz e Reinsel, 2012).') +
#   theme_ipea(legend.position="none",expand_x_limit = F,
#              x_breaks = 10, y_breaks = 4,
#              include_x_text_title = F,include_y_text_title = F)

## ----eval=FALSE, message=FALSE, warning=FALSE, results='hide'-----------------
# png(paste0(getwd(),"/prints_ipeaplot/grafico4.png"),  width=724, height=546)
# print(graf4)
# dev.off()
# 
# 

## ----echo=FALSE, message=FALSE, warning=FALSE, fig.align="center", out.width = "100%"----
#  knitr::include_graphics(paste0(getwd(),"/prints_ipeaplot/grafico4.png"))

## ----echo=FALSE, message=FALSE, warning=FALSE, fig.align="center", fig.width=6, fig.height=6----
# knitr::include_graphics(paste0(getwd(),"/prints_original/grafico5.png"))

## ----eval=FALSE, message=FALSE, warning=FALSE---------------------------------
# graf5 <- ggplot(base_graf5, aes(Ano, value, group=variable, color=variable))+
#   geom_line(linewidth=1)+
#   geom_point(aes(shape=variable, fill = variable),
#              size=3, stroke=1.5, show.legend = F)+
#   labs(color = "",
#        title="GRÁFICO 5",
#        subtitle="Crescimento anual do PIB da União Europeia (2004-2013) (Em %)",
#        caption = 'Fonte: World Bank (2014).\nTradução dos autores.\nObs.: Os dados foram coletados no sítio do Banco Mundial.\nDisponível em: <http://goo.gl/IvtZ1a>. Acesso em: 26 set. 2014.') +
#   theme_ipea(x_breaks = 10, y_breaks = 10, legend.position = 'bottom',
#              expand_y_limit = T,expand_x_limit = F,
#              include_x_text_title = F,include_y_text_title = F) +
#   scale_color_ipea(palette_direction = -1, ) +
#   scale_shape_manual(values = c(19, 21)) +
#   scale_fill_manual(values = c(NA,"white"))

## ----eval=FALSE, message=FALSE, warning=FALSE, results='hide'-----------------
# png(paste0(getwd(),"/prints_ipeaplot/grafico5.png"),  width=724, height=546)
# print(graf5)
# dev.off()
# 

## ----echo=FALSE, message=FALSE, warning=FALSE, fig.align="center", out.width = "100%"----
#  knitr::include_graphics(paste0(getwd(),"/prints_ipeaplot/grafico5.png"))

## ----echo=FALSE, message=FALSE, warning=FALSE, fig.align="center", fig.width=6, fig.height=6----
# knitr::include_graphics(paste0(getwd(),"/prints_original/grafico6.png"))

## ----eval=FALSE, message=FALSE, warning=FALSE---------------------------------
# base_graf6$pais<- factor(base_graf6$pais, levels=rev(sort(base_graf6$pais)))
# graf6 <- ggplot(data=base_graf6, aes(x=pais, y=petroleo)) +
#   geom_bar(stat="identity", fill = '#015f96') +
#   coord_flip() +
#   labs(fill = "",
#        title="GRÁFICO 6",
#        subtitle="Importações de energia em termos do consumo total\nde energia dos países da União Europeia",
#        caption = 'Fonte: Eurostat (2014).\nTradução dos autores.') +
#   theme_ipea(legend.position = 'none', expand_x_limit = F,
#              include_x_text_title = F,include_y_text_title = F)

## ----eval=FALSE, message=FALSE, warning=FALSE, results='hide'-----------------
# png(paste0(getwd(),"/prints_ipeaplot/grafico6.png"),  width=724, height=546)
# print(graf6)
# dev.off()
# 
# 

## ----echo=FALSE, message=FALSE, warning=FALSE, fig.align="center", out.width = "100%"----
#  knitr::include_graphics(paste0(getwd(),"/prints_ipeaplot/grafico6.png"))

## ----echo=FALSE, message=FALSE, warning=FALSE, fig.align="center", fig.width=6, fig.height=6----
# knitr::include_graphics(paste0(getwd(),"/prints_original/grafico7.png"))

## ----eval=FALSE, message=FALSE, warning=FALSE---------------------------------
# base_graf7<- melt(base_graf7, id=c("pais"))
# 
# graf7 <- ggplot(data=base_graf7, aes(x=pais, y=value, group=variable, fill=variable)) +
#   geom_bar(position='dodge',stat="identity") +
#   coord_flip()+
#   labs(fill = "",
#        title="GRÁFICO 7",
#        subtitle="Mudanças nos níveis de desigualdade do BRICS, calculadas pelo coeficiente\nGini, entre o início da década de 1990 e o final da década de 2000",
#        caption = '') +
#   theme_ipea(legend.position = 'bottom',
#              include_x_text_title = F,include_y_text_title = F) +
#   scale_fill_ipea()

## ----eval=FALSE, message=FALSE, warning=FALSE, results='hide'-----------------
# png(paste0(getwd(),"/prints_ipeaplot/grafico7.png"),  width=724, height=546)
# print(graf7)
# dev.off()
# 

## ----echo=FALSE, message=FALSE, warning=FALSE, fig.align="center", out.width = "100%"----
#  knitr::include_graphics(paste0(getwd(),"/prints_ipeaplot/grafico7.png"))

## ----echo=FALSE, message=FALSE, warning=FALSE, fig.align="center", fig.width=6, fig.height=6----
# knitr::include_graphics(paste0(getwd(),"/prints_original/grafico8.png"))

## ----eval=FALSE, message=FALSE, warning=FALSE---------------------------------
# ######## GRAFICO 8
# base_graf8_aux <- base_graf8[,c(1,4)]
# base_graf8_aux$n <- 1:nrow(base_graf8_aux)
# base_graf8_new <- base_graf8[,-4]
# base_graf8_new<- melt(base_graf8_new, id=c("pais"))
# 
# base_graf8_new <- base_graf8_new %>%
#   group_by(variable) %>%
#   mutate(n = 1:n())
# 
# graf8 <-
# ggplot() +
#   geom_bar(data=base_graf8_new, aes(x = reorder(pais, n), y=value,
#                                     group=variable, fill=variable),
#            stat="identity", width = 0.5) +
#   labs(fill = "",
#        title="GRÁFICO 8",
#        subtitle="Crescimento médio real do PIB em PPC (2011-2050) (Em %)",
#        caption = 'Fonte: PwC (2014).\nTradução dos autores') +
#   geom_line(data=base_graf8_aux, aes(x = reorder(pais, n) ,y=`crescimento do PIB (ppc)_perct`, group = 1),
#             color = 'gray') +
#   scale_fill_ipea() +
#   theme_ipea(legend.position = 'bottom', x_text_angle = 90) +
#   coord_cartesian(clip = "off")

## ----eval=FALSE, message=FALSE, warning=FALSE, results='hide'-----------------
# png(paste0(getwd(),"/prints_ipeaplot/grafico8.png"),  width=724, height=546)
# print(graf8)
# dev.off()
# 

## ----echo=FALSE, message=FALSE, warning=FALSE, fig.align="center", out.width = "100%"----
#  knitr::include_graphics(paste0(getwd(),"/prints_ipeaplot/grafico8.png"))

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = identical(tolower(Sys.getenv("NOT_CRAN")), "true"),
  out.width = "100%"
)

# Function that aligns color palettes
pal <- function(col, border = "light gray") {
  n <- length(col)
  plot(0, 0, type="n", xlim = c(0, 1), ylim = c(0, 1), axes = FALSE,
       xlab = "", ylab = "")
  rect(0:(n-1)/n, 0, 1:n/n, 1, col = col, border = border)
}

## ----eval=FALSE, message=FALSE, warning=FALSE---------------------------------
# # Development version
# utils::remove.packages('ipeaplot')
# remotes::install_github("ipeadata-lab/ipeaplot")

## ----eval=TRUE, message=FALSE, warning=FALSE, results='hide'------------------
# Load packages
library(ipeaplot)
library(ggplot2)
library(dplyr)
library(abjData)
library(geobr)
library(patchwork)

# Load mtcars dataset
data(mtcars)

## ----message=FALSE, warning=FALSE, fig.align="center", out.width = "100%"-----
# fig_raw <- ggplot() +
#               geom_point(data = mtcars, aes(x = hp , y = mpg, color = cyl)) +
#               labs(y='Consumo de Combustível (milhas por galão)',
#                    x ='Potência (Número de cavalos)',
#                    color='Cilindradas')
# 
# fig_base <- fig_raw +
#               scale_color_ipea() +
#               theme_ipea()
# 
# fig_base
# 

## ----eval=TRUE, echo=FALSE, fig.dim = c(5, 4),fig.align = "center", out.width = "50%"----
# Salvar as configurações originais
old.par <- par(no.readonly = TRUE)

# Modificar as configurações conforme necessário
par(mfrow = c(3, 1), mar = c(1, 0, 2, 0))
pal(ipea_pal(palette = "Blue")(9)); mtext("Blue")
pal(ipea_pal(palette = "Green")(9)); mtext("Green")
pal(ipea_pal(palette = "Orange")(9)); mtext("Orange")

# Restaurar as configurações originais
par(old.par)

## ----eval=TRUE, echo=FALSE, fig.dim = c(5, 4),fig.align = "center", out.width = "50%"----
# Salvar as configurações originais
old.par <- par(no.readonly = TRUE)

# Modificar as configurações conforme necessário
par(mfrow = c(4, 1), mar = c(1, 0, 2, 0))
pal(ipea_pal(palette = "Red-Blue")(9)); mtext("Red-Blue")
pal(ipea_pal(palette = "Orange-Blue")(9)); mtext("Orange-Blue")
pal(ipea_pal(palette = "Green-Blue")(9)); mtext("Green-Blue")

# Restaurar as configurações originais
par(old.par)

## ----message=FALSE, warning=FALSE, fig.align="center", out.width = "100%"-----
# # paleta sequencial verde
# fig_base + scale_color_ipea(palette = "Green")

## ----message=FALSE, warning=FALSE, fig.align="center", out.width = "100%"-----
# # paleta divergente de laranja a azul
# fig_base + scale_color_ipea(palette = "Orange-Blue")

## ----eval=TRUE----------------------------------------------------------------
df <- abjData::pnud_muni

## ----message=FALSE, warning=FALSE, fig.align="center", out.width = "100%"-----
# # cria variavel identificando a regiao de cada municipio
# df <- df |>
#       mutate(regiao = substring(uf, 1, 1),
#              regiao = case_when(regiao == 1 ~ 'Norte',
#                                 regiao == 2 ~ 'Nordeste',
#                                 regiao == 3 ~ 'Sudeste',
#                                 regiao == 4 ~ 'Sul',
#                                 regiao == 5 ~ 'Centro Oeste'))
# 
# # calcula media de colega de esgoto por ano e regiao
# df_fig1 <- df |>
#            mutate(regiao = substring(uf, 1, 1)) |>
#            group_by(ano, regiao) |>
#            summarise( t_lixo = weighted.mean(x=t_lixo, w = pop)) |>
#            collect()
# 
# # plot
# ggplot() +
#   geom_line(data = df_fig1, aes(x=ano, y=t_lixo, color= regiao)) +
#   scale_color_ipea(palette = 'Orange') +
#   labs(title = 'Proporção da população com coleta de lixo', color='Região') +
#   ylab('Proporção em %') +
#   xlab('Ano') +
#   theme_ipea()
# 

## ----message=FALSE, warning=FALSE, fig.align="center", out.width = "100%"-----
# ggplot() +
#   geom_col(data = df_fig1, aes(x=ano, y=t_lixo, fill= factor(ano))) +
#   scale_fill_ipea(palette = 'Green') +
#   labs(title = 'Proporção da população com coleta de lixo', fill='Ano') +
#   ylab('Proporção em %') +
#   xlab('Ano') +
#   facet_wrap(. ~ regiao) +
#   theme_ipea(x_breaks = 3)
# 

## ----eval=TRUE, message=FALSE, warning=FALSE, results='hide'------------------
# Load municipality and state spatial data
mun <- geobr::read_municipality(year = 2010)
uf  <- geobr::read_state(year = 2010)

## ----eval=TRUE, message=FALSE, warning=FALSE, results='hide', eval = all(!is.null(mun), !is.null(uf))----
# Subset and select specific columns from the 'pnud_muni' dataset
df_escola <- df |>
             subset(ano == 2010) %>%
             select(ano, code_muni = codmun7, e_anosestudo)

# Perform a left join between the 'mun' and 'pnud' data frames
df3 <- dplyr::left_join(mun, df_escola, by = 'code_muni')


## ----message=FALSE, warning=FALSE, fig.align="center", out.width = "100%", eval = all(!is.null(mun), !is.null(uf))----
ggplot() +
  geom_sf(data = df3, aes(fill = e_anosestudo), color = NA) +
  geom_sf(data = uf, color = "black", fill = NA) +
  ggtitle("Média de anos de estudo") +
  scale_fill_ipea(palette = 'Orange-Blue',
                  name='Anos de\nestudo') +
  theme_ipea(axis_lines = 'none', include_ticks = F, axis_values = F)

## ----eval=FALSE, echo=TRUE----------------------------------------------------
# # Salvar apenas em EPS (formato padrão)
# save_ipeaplot(fig_base, file.name = "figura_exemplo")
# 
# # Salvar em múltiplos formatos
# save_ipeaplot(fig_base,
#               file.name = "figura_exemplo",
#               format = c("eps", "pdf", "png"))

## ----eval=FALSE, echo=TRUE----------------------------------------------------
# # Exemplo com configurações personalizadas
# save_ipeaplot(fig_base,
#               file.name = "figura_personalizada",
#               format = c("eps", "png"),
#               width = 8, height = 5,
#               units = "in",
#               dpi = 600,
#               background = "white",
#               path = "figuras/")
# 
# # Salvar com data no nome do arquivo
# save_ipeaplot(fig_base,
#               file.name = "figura_com_data",
#               format = "pdf",
#               include_date = TRUE)
# 
# # Evitar sobrescrever arquivos existentes
# save_ipeaplot(fig_base,
#               file.name = "figura_segura",
#               format = "png",
#               overwrite = FALSE)

# Samples `n` colours, evenly spaced, along a Lab-space spline through `anchors`.
# Used to build the fixed 6-shade families behind the 'NT-Categorical' and
# 'TD-Categorical' qualitative palettes.
ipea_lab_ramp <- function(anchors, n) {
  fn <- grDevices::colorRamp(anchors, space = "Lab", interpolate = "spline")
  cols <- fn(seq(0, 1, length.out = n)) / 255
  toupper(grDevices::rgb(cols[, 1], cols[, 2], cols[, 3]))
}

#' Ipea Color Palette and Scales
#'
#' A 9-color Ipea palette.
#'
#' @description This function creates a vector of n equally spaced colors
#'               along the selected color map.
#'
#' @param n The number of colors (\eqn{\ge 1}) used in the palette.
#' @param alpha The alpha transparency in a number between `0` and `1`.
#' @param begin The (corrected) hue in a number between `0` and `1` at which the
#'        color map begins.
#' @param end The (corrected) hue in a number between `0` and `1` at which the
#'        color map ends.
#' @param palette_direction Sets the order of colors in the scale. If `1`, the default,
#'        colors are ordered from darkest to lightest. If `-1`, the order of
#'        colors is reversed.
#' @param palette A character string indicating the color map option to use.
#' These options are available:'Blue', 'Green', 'Orange', 'Pink', 'Pink-Deep', 'Red-Blue'
#' 'Orange-Blue', 'Green-Blue', 'Red-Blue-White', 'Orange-Blue-White',
#' 'Green-Blue-White', 'Viridis', 'Inferno', 'Magma', 'Plasma', 'Cividis',
#' 'NT', 'NT-Categorical', 'TD', 'TD-Categorical'.
#'
#' @references
#' 'Blue','Green','Orange','Pink','Pink-Deep','Green-Blue','Green-Blue-White','Red-Blue','Red-Blue-White',
#' 'Orange-Blue','Orange-Blue-White', 'Viridis','Inferno',
#' 'Magma','Plasma' and 'Cividis': https://pmassicotte.github.io/paletteer_gallery/
#' 'NT' and 'TD' are sequential palettes taken from the cover, section-title and
#' chart colours of two Ipea editorial series (Nota Tecnica and Texto para
#' Discussao, respectively). 'NT-Categorical' and 'TD-Categorical' are qualitative
#' 12-colour sets built from the same two publications, interleaving 6 shades of
#' each of their two hues so that adjacent categories stay visually distinct.
#' `scale_color_ipea()`/`scale_fill_ipea()` switch automatically from 'NT'/'TD'
#' to their '-Categorical' variant when the mapped variable is discrete, so most
#' users never need to type the '-Categorical' suffix themselves.
#'
#' @return \code{ipea_palette} produces a character vector, \code{cv}, containing color hex codes.
#' This vector can be utilized to establish a custom color scheme for future graphics using \code{palette(cv)},
#' or it can be applied directly as a \code{col =} parameter in graphic functions or within \code{par}.
#'
ipea_palette <- function(palette = c('Blue','Green','Orange','Pink','Pink-Deep','Green-Blue','Green-Blue-White','Red-Blue','Red-Blue-White',
                                     'Orange-Blue','Orange-Blue-White', 'Viridis','Inferno','Magma','Plasma','Cividis',
                                     'NT','NT-Categorical','TD','TD-Categorical'),
                         n,
                         alpha = 1,
                         begin = 0,
                         end = 1,
                         palette_direction = 1) {

  palette <-  ifelse(missing(palette),'Blue',palette)

  if (begin < 0 | begin > 1 | end < 0 | end > 1) {
    stop("begin and end must be in a number between `0` and `1`")
  }

  if (abs(palette_direction) != 1) {
    stop("palette_direction must be 1 or -1")
  }

  if (n == 0) {
    return(character(0))
  }

  if (palette_direction == -1) {
    tmp <- begin
    begin <- end
    end <- tmp
  }

  if (palette ==  "Blue") {
    # Set colours to the Blue palette using manual_pal function
    colours <- paletteer::paletteer_c("ggthemes::Blue", n)
  } else if (palette ==  "Green") {
    # Set colours to the Green palette using manual_pal function
    colours <-  paletteer::paletteer_c("ggthemes::Green", n)
  } else if (palette ==  "Orange") {
    # Set colours to the Orange palette using manual_pal function
    colours <- paletteer::paletteer_c("ggthemes::Orange", n)
  } else if (palette ==  "Pink") {
    # Set colours to the Pink palette using manual_pal function
    colours <- paletteer::paletteer_dynamic("cartography::pink.pal", n)
  } else if (palette ==  "Pink-Deep") {
    # Set colours to the Pink-Deep palette using manual_pal function
    colours <- rev(paletteer::paletteer_d("RColorBrewer::RdPu", n)[-(1:2)])
  } else if (palette ==  "Red-Blue") {
    # Set colours to the Red-Blue palette using manual_pal function
    colours <- paletteer::paletteer_c("ggthemes::Red-Blue Diverging", n)
  } else if (palette ==  "Green-Blue") {
    # Set colours to the Green-Blue palette using manual_pal function
    colours <- paletteer::paletteer_c("ggthemes::Green-Blue Diverging", n)
  } else if (palette ==  "Orange-Blue") {
    # Set colours to the Orange-Blue palette using manual_pal function
    colours <- paletteer::paletteer_c("ggthemes::Orange-Blue Diverging", n)
  } else if (palette ==  "Red-Blue-White") {
    # Set colours to the Red-Blue palette using manual_pal function
    colours <- paletteer::paletteer_c("ggthemes::Red-Blue-White Diverging", n)
  } else if (palette ==  "Green-Blue-White") {
    # Set colours to the Green-Blue palette using manual_pal function
    colours <- paletteer::paletteer_c("ggthemes::Green-Blue-White Diverging", n)
  } else if (palette ==  "Orange-Blue-White") {
    # Set colours to the Orange-Blue palette using manual_pal function
    colours <- paletteer::paletteer_c("ggthemes::Orange-Blue-White Diverging", n)
  } else if (palette ==  "Viridis") {
    # Set colours to the Viridis palette using manual_pal function
    colours <- paletteer::paletteer_c("viridis::viridis", n)
  } else if (palette ==  "Inferno") {
    # Set colours to the Inferno palette using manual_pal function
    colours <- paletteer::paletteer_c("viridis::inferno", n)
  } else if (palette ==  "Magma") {
    # Set colours to the Magma palette using manual_pal function
    colours <- paletteer::paletteer_c("viridis::magma", n)
  } else if (palette ==  "Plasma") {
    # Set colours to the Plasma palette using manual_pal function
    colours <- paletteer::paletteer_c("viridis::plasma", n)
  } else if (palette ==  "Cividis") {
    # Set colours to the Cividis palette using manual_pal function
    colours <- paletteer::paletteer_c("viridis::cividis", n)
  } else if (palette == "NT") {
    # Sequential palette from the Ipea "Nota Tecnica" series: cover background,
    # section-title colour and the "Depois das transferencias" line in Grafico 6
    # of NT_Disoc_120_Pobreza_e_desigualdade_no_Brasil.pdf (dark -> signature -> light).
    colours <- c("#3E0C2C", "#9C2565", "#E8B8D3")
  } else if (palette == "NT-Categorical") {
    # Qualitative set for the "NT" family: 6 magenta shades (as above) interleaved
    # with 6 blue shades (the contrast colour used for "Antes das transferencias"
    # in the same chart), so consecutive categories alternate hue.
    magenta6 <- ipea_lab_ramp(c("#3E0C2C", "#9C2565", "#E8B8D3"), 6)
    blue6    <- ipea_lab_ramp(c("#0B2E5C", "#2571B8", "#B8D9F0"), 6)
    colours  <- rep_len(as.vector(rbind(magenta6, blue6)), n)
  } else if (palette == "TD") {
    # Sequential palette from the Ipea "Texto para Discussao" series: cover
    # background and the "TEXTO para DISCUSSAO" title-bar colour (dark -> signature -> light).
    colours <- c("#0B2E5C", "#2571B8", "#B8D9F0")
  } else if (palette == "TD-Categorical") {
    # Qualitative set for the "TD" family: 6 blue shades (as above) interleaved
    # with 6 grey shades, mirroring the blue/grey pairing used across the six
    # series of Grafico 1 (p. 22) of TD_3203_Web.pdf.
    blue6 <- ipea_lab_ramp(c("#0B2E5C", "#2571B8", "#B8D9F0"), 6)
    grey6 <- ipea_lab_ramp(c("#333333", "#E0E0E0"), 6)
    colours <- rep_len(as.vector(rbind(blue6, grey6)), n)
  } else {
    # Stop the execution and display an error message if palettes is none of the specified values
    stop("Palette palettes must be 'Blue','Green','Orange','Pink','Pink-Deep','Green-Blue','Green-Blue-White','Red-Blue','Red-Blue-White',
                                   'Orange-Blue','Orange-Blue-White', 'Viridis','Inferno','Magma','Plasma','Cividis',
                                   'NT','NT-Categorical','TD','TD-Categorical'")
  }

  map_cols <- as.character(colours)
  # Keep just the opaque "#RRGGBB" part (7 characters): colorRamp() ignores
  # alpha, so any trailing alpha byte (e.g. paletteer's "#RRGGBBAA") is simply
  # dropped rather than overwritten. Using substr(...,1,6) here used to keep
  # only 5 hex digits after "#" and pad a literal "F", which silently forced
  # the low nibble of the blue channel to F in every palette.
  map_cols <- substr(map_cols, 1, 7)
  fn_cols <- grDevices::colorRamp(map_cols, space = "Lab", interpolate = "spline")
  cols <- fn_cols(seq(begin, end, length.out = n)) / 255
  grDevices::rgb(cols[, 1], cols[, 2], cols[, 3], alpha = alpha)
}

# Evaluates all arguments (see #81)
force_all <- function(...) list(...)

#' Ipea palette
#'
#' @param alpha	The alpha transparency in a number between `0` and `1`.
#' @param begin The (corrected) hue in a number between `0` and `1` at which the color map begins.
#' @param end The (corrected) hue in a number between `0` and `1` at which the color map ends.
#' @param palette_direction Sets the order of colors in the scale. If 1, the default,
#'        colors are ordered from darkest to lightest. If -1, the order of
#'        colors is reversed.
#' @param palette A character string indicating the color map option to use.
#'  These options are available: 'Blue', 'Green', 'Orange', 'Pink', 'Pink-Deep', 'Red-Blue'
#'  'Orange-Blue', 'Green-Blue', 'Viridis', 'Inferno', 'Magma', 'Plasma'
#'  'Cividis', 'NT', 'NT-Categorical', 'TD', 'TD-Categorical'.
#' @references
#'  'Blue','Green','Orange','Pink','Pink-Deep','Green-Blue','Green-Blue-White','Red-Blue','Red-Blue-White',
#'  'Orange-Blue','Orange-Blue-White', 'Viridis','Inferno',
#'  'Magma','Plasma' and 'Cividis': https://pmassicotte.github.io/paletteer_gallery/
#'  'NT'/'TD' and their '-Categorical' variants: see `ipea_palette()`.
#'
#' @return \code{ipea_palette} produces a character vector, \code{cv}, containing color hex codes.
#' This vector can be utilized to establish a custom color scheme for future graphics using \code{palette(cv)},
#' or it can be applied directly as a \code{col =} parameter in graphic functions or within \code{par}.
#'
#' @export
#' @examples
#' scales::show_col(ipea_pal()(10))
#' scales::show_col(ipea_pal(palette_direction = -1)(6))
#' scales::show_col(ipea_pal(begin = 0.2, end = 0.8)(4))
#' scales::show_col(ipea_pal(palette = "Green")(6))
#' scales::show_col(ipea_pal(palette = "NT")(10))
#' scales::show_col(ipea_pal(palette = "TD-Categorical")(12))

ipea_pal <- function(palette = c('Blue','Green','Orange','Pink','Pink-Deep','Green-Blue','Green-Blue-White','Red-Blue','Red-Blue-White',
                                 'Orange-Blue','Orange-Blue-White', 'Viridis','Inferno','Magma','Plasma','Cividis',
                                 'NT','NT-Categorical','TD','TD-Categorical'),
                     alpha = 1, begin = 0, end = 1, palette_direction = 1) {
  force_all(palette, alpha, begin, end, palette_direction)
  function(n) {
    ipea_palette(palette, n, alpha, begin, end, palette_direction)
  }
}

all_palettes <- c(
  "Blue", "Green", "Orange", "Pink", "Pink-Deep",
  "Green-Blue", "Green-Blue-White", "Red-Blue", "Red-Blue-White",
  "Orange-Blue", "Orange-Blue-White",
  "Viridis", "Inferno", "Magma", "Plasma", "Cividis",
  "NT", "NT-Categorical", "TD", "TD-Categorical"
)

test_that("ipea_palette() returns n valid hex colours for every named palette", {
  for (p in all_palettes) {
    cols <- ipea_palette(palette = p, n = 5)
    expect_length(cols, 5)
    expect_true(all(grepl("^#[0-9A-Fa-f]{6,8}$", cols)), info = p)
  }
})

test_that("ipea_palette() returns character(0) for n = 0", {
  expect_identical(ipea_palette(palette = "Blue", n = 0), character(0))
})

test_that("ipea_palette() errors on an unknown palette name", {
  expect_error(ipea_palette(palette = "NaoExiste", n = 3), "Palette")
})

test_that("ipea_palette() validates begin/end", {
  expect_error(ipea_palette(palette = "Blue", n = 3, begin = -1), "begin and end")
  expect_error(ipea_palette(palette = "Blue", n = 3, end = 2), "begin and end")
})

test_that("ipea_palette() validates palette_direction", {
  expect_error(ipea_palette(palette = "Blue", n = 3, palette_direction = 2), "palette_direction")
})

test_that("NT/TD sequential palettes return n colours interpolated from their anchors", {
  expect_length(ipea_palette(palette = "NT", n = 10), 10)
  expect_length(ipea_palette(palette = "TD", n = 10), 10)
})

test_that("NT-Categorical/TD-Categorical recycle to any n", {
  expect_length(ipea_palette(palette = "NT-Categorical", n = 12), 12)
  expect_length(ipea_palette(palette = "NT-Categorical", n = 20), 20)
  expect_length(ipea_palette(palette = "TD-Categorical", n = 5), 5)
})

test_that("the blue-channel low-nibble colour bug stays fixed", {
  # Regression test: an incorrect string offset used to silently force the
  # low nibble of the blue channel to 'F' in every generated colour.
  cols <- ipea_palette(palette = "Blue", n = 9)
  blue_low_nibble <- toupper(substr(cols, 7, 7))
  expect_false(all(blue_low_nibble == "F"))
})

test_that("ipea_pal() returns a palette-generating function", {
  f <- ipea_pal(palette = "Green")
  expect_type(f, "closure")
  expect_length(f(4), 4)
})

#' @title Font size for data labels in Ipea graphics
#'
#' @description Converts a font size in points (pt) into the `size`
#' argument/aesthetic (in mm) expected by `ggplot2::geom_text()`,
#' `ggplot2::geom_label()` and similar geoms. `ggplot2` specifies text size
#' for geoms in mm and internally converts it to points by multiplying by
#' `2.845276` (`72.27/25.4`); this helper performs the inverse conversion so
#' that labels are rendered at an exact point size.
#'
#' @param pt Numeric. Desired font size in points. Defaults to `6`, the size
#'        used by the editorial guidelines of the Institute for Applied
#'        Economic Research (Ipea) for labels drawn over data (e.g. values on
#'        bars or lines). See
#'        <https://www.ipea.gov.br/manualeditorial/padroes/padroes-grafico-visuais/ilustracoes-pi.html>.
#'
#' @return A numeric value (in mm) to be passed to the `size`
#'         argument/aesthetic of `ggplot2::geom_text()`, `ggplot2::geom_label()`
#'         or similar geoms.
#' @export
#' @family ggplot2 theme functions
#'
#' @examples
#' library(ggplot2)
#' ggplot(mtcars[1:5, ], aes(hp, mpg, label = round(mpg))) +
#'   geom_text(size = ipea_label_size(), colour = "black") +
#'   theme_ipea()
ipea_label_size <- function(pt = 6) {
  checkmate::assert_number(pt, lower = 0)
  pt / 2.845276
}

test_that("ipea_label_size() converts pt to the mm size ggplot2 expects", {
  expect_equal(ipea_label_size(6), 6 / 2.845276)
  expect_equal(ipea_label_size(7), 7 / 2.845276)
})

test_that("ipea_label_size() defaults to 6pt", {
  expect_equal(ipea_label_size(), ipea_label_size(6))
})

test_that("ipea_label_size() rejects negative sizes", {
  expect_error(ipea_label_size(-1))
})

test_that("ipea_label_size() accepts zero", {
  expect_equal(ipea_label_size(0), 0)
})

## Exemplos de acordo com site
# https://www.ipea.gov.br/sites/manualeditorial/padroes-editoriais/padroes-grafico-visuais/ilustracoes/graficos
remotes::install_github("ipeadata-lab/ipeaplot")

# checks spelling
library(spelling)
devtools::spell_check(pkg = ".", vignettes = TRUE, use_wordlist = TRUE)

# Update documentation
devtools::document(pkg = ".")

devtools::check(pkg = ".",  cran = FALSE, env_vars = c(NOT_CRAN = "true"))

library(ipeaplot)
library(ggplot2)
library(dplyr)
extrafont::fonttable()

# Teste paleta
Green       = paletteer::paletteer_d("ggsci::green_material")
Orange      = paletteer::paletteer_d("ggsci::orange_material")
Blue        = paletteer::paletteer_d("ggsci::blue_material")
Pink        = paletteer::paletteer_d("ggsci::pink_material")
Green_Blue  = paletteer::paletteer_c("ggthemes::Classic Green-Blue", 10)
Orange_Blue = paletteer::paletteer_c("ggthemes::Classic Orange-Blue", 10)
Red_Blue    = paletteer::paletteer_c("ggthemes::Classic Red-Blue", 10)

library(colorblindcheck)
palette_dist(Green_Blue)
palette_check(Green_Blue, plot = TRUE)
palette_check(Orange_Blue, plot = TRUE)
palette_check(Red_Blue, plot = TRUE)

library(showtext)


## Exemplo 1
graph <- abjData::pnud_uf %>%
  filter(substr(uf,1,1) == "3") %>%
  select(1:4)


ggplot(base_graf2, aes(x='', y=percentual, fill=Assunto))+
  geom_bar(width = 1, stat = "identity", color= 'black')+
  coord_polar("y", start=0) +
  labs(x="",
       y="",
       fill = "",
       title="GRÁFICO 2",
       subtitle="Assuntos das manifestações recebidas pela Ouvidoria do Ipea (2004-2014) (Em %)",
       caption = 'Elaboração dos autores.') +
  theme_ipea(legend.position="bottom", axis = 'none', axis_values = F, pie.adjust = 'pie') +
  scale_fill_ipea(discrete = T,palette = 'Orange-Blue') +
  insert_text(label = 'percentual', show_percents = T, pie_plot = T)

# Create a discrete scatter plot with 'mpg' on the x-axis, 'wt' on the y-axis, and filled by 'quantile'
# Use the 'scale_ipea()' function to apply the IPEA discrete fill scale
ggplot(data = graph, aes(x = ufn,
                         y = espvida,
                         fill = ano)) +
  geom_bar(stat="identity", width= 0.5) +
  scale_fill_ipea(discrete = F) +
  ggplot2::geom_text(
    aes(label = paste0(gsub("\\.", ",", round(get(label), decimals)))
    ))

  insert_text(label = 'espvida')
  labs(x="",
       y="",
       fill = "",
       title="GRÁFICO 5",
       subtitle="Indicadores de infraestrutura das escolas - capitais regionais do Nordeste (2018)",
       caption = 'Fonte: ipea') +
  theme_ipea(legend.position = 'bottom', geom = 'bar')


# Exemplo 2
graph <- abjData::pnud_uf %>%
  filter(substr(uf,1,1) == "2", ano == 2010) %>%
  select(1:3,pop) %>%
  mutate(share = (pop/sum(pop))*100)


# Barplot
bp <- ggplot(graph, aes(x="", y=share, fill=ufn))+
  geom_bar(width = 1, stat = "identity") +
  coord_polar("y",start=0) +
  scale_fill_ipea(discrete = T, palette = 'Blue') +
  labs(x="",
       y="",
       fill = "",
       title="GRÁFICO 5",
       subtitle="Indicadores de infraestrutura das escolas - capitais regionais do Nordeste (2018)",
       caption = 'Fonte: ipea') +
  theme_ipea(legend.position = 'bottom')
bp

# 1) Mesmo comportamento (só JPG, 300 dpi)
#    (pode passar com ou sem extensão; a função remove se vier)
save_ipeaplot(
  gplot  = bp,
  file.name = "arquivo",          # "arquivo.jpg" também funciona
  format = "jpg",
  dpi    = 300
)

# 2) Salvar em múltiplos formatos de uma vez
save_ipeaplot(
  bp, "arquivo",
  format = c("jpg","png","pdf","eps"),
  dpi = 300
)

# 3) Salvar dentro de uma pasta, sem sobrescrever (cria _1, _2, ...)
save_ipeaplot(
  bp, "arquivo",
  format    = c("jpg","png"),
  path      = "results/graphs",
  overwrite = FALSE
)

# 4) Adicionar data ao nome do arquivo (ex.: arquivo_20250910.png)
save_ipeaplot(
  bp, "arquivo",
  format       = "png",
  include_date = TRUE
)

# 5) PNG transparente em alta resolução
save_ipeaplot(
  bp, "arquivo",
  format     = "png",
  background = NA,
  dpi        = 600
)


# Exemplo 3
graph <- abjData::pnud_uf %>% filter(ano == 2010)

ggplot(graph, aes(x = ufn, y = rdpc))+
  geom_bar(stat = "identity", fill = 'blue', width= 0.5) +
  insert_text(label = "rdpc", vertical = F) +
  labs(x="",
       y="",
       fill = "",
       title="GRÁFICO 5",
       subtitle="Indicadores de infraestrutura das escolas - capitais regionais do Nordeste (2018)",
       caption = 'Fonte: ipea') +
  theme_ipea(legend.position = 'bottom', geom = 'bar', direction = 'horizontal')



# Exemplo 4
# populacao
options(scipen = 999)
pop <- sidrar::get_sidra(6579, period = c(paste0(2000:2020)), geo = "Brazil")
pop <- janitor::clean_names(pop)
pop <- dplyr::select(pop, ano , pop = valor)



ggplot(pop, aes(x = ano, y = pop/1000000))+
  geom_bar(stat = "identity", fill = 'blue', width= 0.5) +
  labs(x="",
       y="",
       fill = "",
       title="GRÁFICO 5",
       subtitle="Indicadores de infraestrutura das escolas - capitais regionais do Nordeste (2018)",
       caption = 'Fonte: ipea') +
  theme_ipea(legend.position = 'bottom', geom = 'bar')


# Exemplo 5
# populacao
options(scipen = 999)
pop <- sidrar::get_sidra(6579, period = c(paste0(2000:2020)), geo = "Brazil")
pop <- janitor::clean_names(pop)
pop <- dplyr::select(pop, ano , pop = valor)

pib <- sidrar::get_sidra(5938, period = c(paste0(2000:2020)), geo = "Brazil")
pib <- janitor::clean_names(pib)
pib <- subset(pib, variavel == "Produto Interno Bruto a preços correntes")
pib <- dplyr::select(pib, ano , pib = valor)

pib_pc <- left_join(pib,pop) %>% mutate(pib_pc = pib/pop) %>% filter(!is.na(pib_pc))

ggplot(pib_pc, aes(x = ano, y = pib_pc))+
  geom_line(fill = 'blue', group = 1) +
  geom_point() +
  labs(x="",
       y="",
       fill = "",
       title="GRÁFICO 5",
       subtitle="Indicadores de infraestrutura das escolas - capitais regionais do Nordeste (2018)",
       caption = 'Fonte: ipea') +
  insert_text(label = 'pib_pc') +
  theme_ipea(legend.position = 'bottom')

# Exemplo 6
graph <- abjData::pnud_uf %>%
  mutate(reg = ifelse(substr(uf,1,1) == 1,'A - Norte',
               ifelse(substr(uf,1,1) == 2,'B - Nordeste',
               ifelse(substr(uf,1,1) == 3,'C - Sudeste',
               ifelse(substr(uf,1,1) == 4,'D - Sul','E  Centro-Oeste'))))) %>%
  group_by(reg,ano) %>% dplyr::summarise(rdpc = weighted.mean(rdpc, w = pop))

ggplot(graph, aes(x = as.character(ano), y = rdpc))+
  geom_bar(stat = "identity", fill = 'blue', width= 0.5) +
  insert_text(label = "rdpc", vertical = T) +
  facet_wrap(.~reg, scales = 'free') +
  labs(x="",
       y="",
       fill = "",
       title="GRÁFICO 5",
       subtitle="Indicadores de infraestrutura das escolas - capitais regionais do Nordeste (2018)",
       caption = 'Fonte: ipea') +
  theme_ipea(legend.position = 'bottom', geom = 'bar')

# Exemplo 7
set.seed(123)
df <- expand.grid(ufn = distinct(abjData::pnud_uf, ufn)$ufn,
                  situacao = c('muito ruim','ruim','mediano','bom','muito bom'))
df$num <- runif(nrow(df))
df <- df %>%
  group_by(ufn) %>%
  mutate(prop = (num/sum(num))*100,
         total = sum(prop))

graph <- arrange(transform(df,
                              Categoria =
                                factor(df$situacao,
                                       levels=c('muito ruim','ruim','mediano','bom','muito bom'))),
                    df$situacao)

graph <- graph %>% mutate(label = paste(round(prop,0)))

ggplot(graph, aes(x = ufn, y = prop, fill = situacao))+
  geom_bar(stat = "identity") +
  insert_text(label = 'label', vertical = F, show_percents = T) +
  scale_fill_ipea(discrete = T) +
  labs(x="",
       y="",
       fill = "",
       title="GRÁFICO 5",
       subtitle="Indicadores de infraestrutura das escolas - capitais regionais do Nordeste (2018)",
       caption = 'Fonte: ipea') +
  theme_ipea(legend.position = 'bottom', geom = 'bar')


# Exemplo 8
graph <- abjData::pnud_muni %>%
  filter(substr(codmun6 ,1,2) == "27", ano == 2010) %>%
  select(code_muni = codmun7, espvida)

mun <- geobr::read_municipality(code_muni = 'AL') %>% left_join(graph)

# Create a discrete scatter plot with 'mpg' on the x-axis, 'wt' on the y-axis, and filled by 'quantile'
# Use the 'scale_ipea()' function to apply the IPEA discrete fill scale
ggplot(data = mun, aes(fill = espvida)) +
  geom_sf() +
  scale_fill_ipea(direction = 'horizontal') +
  labs(x="",
       y="",
       fill = "",
       title="GRÁFICO 5",
       subtitle="Indicadores de infraestrutura das escolas - capitais regionais do Nordeste (2018)",
       caption = 'Fonte: ipea') +
  theme_ipea(legend.position = 'bottom', axis = 'none', text = F)

#' Ggplot theme for Ipea charts and figures
#'
#' @description Applies a custom theme for ggplot figures following the editorial
#' guidelines used by the Institute for Applied Economic Research - Ipea. The
#' function includes standardized formatting of options for axis lines, text,
#'
#' @param axis_lines A character vector specifying the axis style. Valid options are
#'        `"none"` (no axis lines), `"full"` (full-length axis lines), and
#'        `"half"` (half-length axis lines), the default.
#' @param axis_values Logical value indicating whether to show text elements. If `TRUE`,
#'        axis text will be displayed in black; otherwise, they will
#'        be hidden.
#' @param legend.position A character vector specifying the position of the
#'        legend. Valid options are `"bottom"` (default, matching the Ipea
#'        editorial guidelines), `"right"`, `"left"`, and `"top"`.
#' @param grid.adjust Defines whether the grid lines should be `"horizontal"`
#'       (default) or `"vertical"`.
#' @param x_breaks Numeric. The number of breaks on the x-axis
#' @param y_breaks Numeric. The number of breaks on the y-axis
#' @param expand_x_limit Logical value that indicates whether the x-axis
#'        boundary should be expanded. If `TRUE`, the x-axis limits will be
#'        expanded; otherwise there will be no change
#' @param expand_y_limit Logical value that indicates whether the y-axis
#'        boundary should be expanded. If `TRUE`, the x-axis limits will be
#'        expanded; otherwise there will be no change
#' @param x_text_angle Numeric. Angle in degrees of the text in the x-axis.
#' @param include_x_text_title Logical. Whether to include x text title Defaults to `TRUE`.
#' @param include_y_text_title Logical. Whether to include x text title. Defaults to `TRUE`.
#' @param include_ticks Logical. Whether to include ticks. Defaults to `TRUE`.
#' @param ... Additional arguments to be passed to the `theme` function from the
#'        `ggplot2` package.
#'
#' @return A custom theme for IPEA graphics.
#' @import ggplot2 ggthemes rlang
#' @export
#' @family ggplot2 theme functions
#'
#' @examples
#' # Creating theme for ggplot2 graph using default arguments
#' library(ggplot2)
#' fig_raw <- ggplot() +
#'   geom_col(data = mtcars, aes(x = hp , y = mpg, fill = cyl)) +
#'   theme_ipea()
#'




theme_ipea <- function(axis_lines = 'full',
                       axis_values = TRUE,
                       legend.position = 'bottom',
                       grid.adjust = 'horizontal',
                       x_breaks = NULL, # Valor padrão definido como NULL
                       y_breaks = NULL, # Valor padrão definido como NULL
                       expand_x_limit = TRUE,
                       expand_y_limit = TRUE,
                       x_text_angle = 0,
                       include_x_text_title = TRUE,
                       include_y_text_title = TRUE,
                       include_ticks = TRUE,
                       ...){

  structure(list(axis_lines = axis_lines,
                 axis_values = axis_values,
                 legend.position = legend.position, grid.adjust = grid.adjust,
                 x_breaks = x_breaks, y_breaks = y_breaks,
                 expand_x_limit = expand_x_limit,expand_y_limit = expand_y_limit,
                 x_text_angle = x_text_angle,
                 include_x_text_title = include_x_text_title, include_y_text_title = include_y_text_title,
                 include_ticks = include_ticks, ...), class = "scale_auto_ipea")

}

nicelimits <- function(x) {
  range(scales::extended_breaks(only.loose = TRUE)(x))
}

# `element_line(linewidth = x)` do ggplot2 nao produz uma linha de x pt no
# arquivo vetorial final. O motor grafico do R interpreta 1 unidade de "lwd"
# como 1/96 polegada, enquanto ggplot2 converte mm -> pt usando .pt = 72.27/25.4.
# O resultado, validado empiricamente lendo o operador `w` (largura de traco)
# do PDF/EPS gerado, e: largura_final_pt = linewidth * 2.133957.
# Para atender a diretriz do Manual Editorial do Ipea (eixos, fechamento e
# hastes com 0,5 pt): https://www.ipea.gov.br/manualeditorial/padroes/padroes-grafico-visuais/ilustracoes-pi.html
ipea_lwd_from_pt <- function(pt) pt / 2.133957


my_pretty_breaks <- function(x, n_breaks = NULL, sd = 0.05, ...) {
  if (is.null(n_breaks)) {
    n_breaks <- 5
  }

  # Aplica a expansão ao intervalo dos dados
  range_x <- range(x, na.rm = T)

  # Calcula os breaks com base no intervalo fornecido
  breaks <- seq(from = range_x[1], to = range_x[2], length.out = n_breaks)

  # Ajusta precisão para evitar números repetidos
  t <- 0
  len <- length(unique(round(breaks, t)))
  while (len != n_breaks && t < 10) {
    t <- t + 1
    len <- length(unique(round(breaks, t)))
  }

  # Aplicar arredondamento com base em t
  if (t == 0) {
    # Quando t == 0, arredondamos para inteiros
    breaks <- round(breaks, 0)
  } else {
    # Quando t > 0, arredondamos de acordo com a precisão
    breaks <- round(breaks, t)
  }

  # Ajusta para intervalos consistentes, mantendo-os dentro de `range_x`
  intervals <- diff(breaks)
  while (any(sd(intervals) > sd | is.na(sd(intervals))) && t < 10) {
    t <- t + 1
    breaks <- round(seq(from = range_x[1], to = range_x[2], length.out = n_breaks), t)
    intervals <- diff(breaks)
  }

  # Verifica os limites
  if (max(breaks) >= range_x[2] & !(max(breaks) >= -1 & max(breaks) <= 1)) {
    breaks[length(breaks)] <- floor(range_x[2])
  }

  return(breaks)
}



#' @export
#' @method ggplot_add scale_auto_ipea
# Método ggplot_add para a classe 'scale_auto_ipea'
ggplot_add.scale_auto_ipea <- function(object, plot, object_name, ...) {
  # Extração de argumentos do objeto
  args <- object
  axis_lines <- args$axis_lines
  axis_values <- args$axis_values
  legend.position <- args$legend.position
  grid.adjust <- args$grid.adjust
  x_breaks <- args$x_breaks
  y_breaks <- args$y_breaks
  expand_x_limit <- args$expand_x_limit
  expand_y_limit <- args$expand_y_limit
  x_text_angle <- args$x_text_angle
  include_x_text_title <- args$include_x_text_title
  include_y_text_title <- args$include_y_text_title
  include_ticks <- args$include_ticks




  hjust <- ifelse(x_text_angle == 0,0.5, 0.8)
  # Distancia entre hastes (ticks) e dados (rotulos dos eixos): 1mm, conforme
  # o Manual Editorial do Ipea.
  vjust <- ifelse(x_text_angle == 0,1, 1)
  # vjust = 1 ancora o topo do texto logo abaixo da haste; com vjust = -1
  # (usado antes) rotulos de uma linha ficavam desalinhados e podiam colidir
  # com o titulo do eixo sempre que outras categorias do mesmo eixo tivessem
  # rotulos com "\n" (mais de uma linha).
  vjust_angle <- ifelse(x_text_angle == 0,1, 0.5)


  ### check inputs
  checkmate::assert_number(x = x_text_angle)
  checkmate::assert_string(x = axis_lines, pattern = c('none|half|full'))
  checkmate::assert_string(x = grid.adjust, pattern = c('vertical|horizontal'))
  checkmate::assert_string(x = legend.position, pattern = c('right|left|bottom|top|none'))

  color = "gray75"

  # Eixos, fechamento e hastes: 0,5 pt (preto 100%). Grades internas: 0,5 pt (preto 25%).
  lwd_05pt <- ipea_lwd_from_pt(0.5)

  if(grid.adjust == 'horizontal'){
    panel.grid.major.x = ggplot2::element_blank()
    panel.grid.major.y = ggplot2::element_line(colour = color, linewidth = lwd_05pt)

  } else if (grid.adjust == 'vertical'){
    panel.grid.major.x = ggplot2::element_line(colour = color, linewidth = lwd_05pt)
    panel.grid.major.y = ggplot2::element_blank()
  }


  if (axis_lines == "half") {
    # Define the axis line and panel border for "half" style
    axis.line.x = ggplot2::element_line(linewidth = lwd_05pt, linetype = "solid", colour = "black")
    axis.line.y = ggplot2::element_line(linewidth = lwd_05pt, linetype = "solid", colour = "black")
    axis.ticks.x = ggplot2::element_line(colour = "black", linewidth = lwd_05pt)
    axis.ticks.y = ggplot2::element_line(colour = "black", linewidth = lwd_05pt)
    panel.border = ggplot2::element_blank()

  } else if (axis_lines == "full") {
    # Define the axis line and panel border for "full" style
    axis.line.x = ggplot2::element_line(linewidth = lwd_05pt, linetype = "solid", colour = "black")
    axis.line.y = ggplot2::element_line(linewidth = lwd_05pt, linetype = "solid", colour = "black")
    axis.ticks.x = ggplot2::element_line(colour = "black", linewidth = lwd_05pt)
    axis.ticks.y = ggplot2::element_line(colour = "black", linewidth = lwd_05pt)
    panel.border = ggplot2::element_rect(linewidth = lwd_05pt, colour = "black", fill = NA)

  } else if (axis_lines == "none") {
    # Define the axis line and panel border for other styles
    axis.line.x  = ggplot2::element_blank()
    axis.line.y  = ggplot2::element_blank()
    axis.ticks.x = ggplot2::element_blank()
    axis.ticks.y = ggplot2::element_blank()
    axis.text.x  = ggplot2::element_blank()
    axis.text.y  = ggplot2::element_blank()
    panel.grid.major.x = ggplot2::element_blank()
    panel.grid.major.y = ggplot2::element_blank()
    panel.border = ggplot2::element_blank()

  } else {
    stop("Argument axis_lines must be 'half', 'full' or 'none'.")
  }


  if(isFALSE(include_ticks)){
    axis.ticks.x = ggplot2::element_blank()
    axis.ticks.y = ggplot2::element_blank()
    hjust <- ifelse(x_text_angle == 0,0.5, 0.8)
    vjust <- ifelse(x_text_angle == 0,3, 0.5)
    vjust_angle <- ifelse(x_text_angle == 0,1, 0)
  }

  if(axis_values == T){
    # Dados externos (eixos): Frutiger LT Pro 55 roman, tamanho 7, preto 100%
    axis.text.y  = ggplot2::element_text(family = "sans", size = 7, color = 'black',
                                         margin = margin(r = vjust, unit = 'mm'))
    # axis.text.x  = ggplot2::element_text(angle = x_text_angle,  hjust= hjust, margin = margin(b = vjust,unit = 'mm'))
    axis.text.x  = ggplot2::element_text(family = "sans", size = 7, angle = x_text_angle,color = 'black',
                                         hjust = hjust, vjust = vjust_angle,margin = margin(b = vjust,unit = 'mm'))
  } else if(axis_values == F){
    axis.text.x  = ggplot2::element_blank()
    axis.text.y  = ggplot2::element_blank()
  } else {
    stop("Argument axis_values must be TRUE or FALSE")
  }


  if(include_x_text_title == T){
    axis.title.x  = ggplot2::element_text(family = "sans", size = 7, color = 'black',margin = margin(t = 4, r = 0, b = 0, l = 0, unit = 'mm'))
  } else if(include_x_text_title == F){
    axis.title.x  = ggplot2::element_blank()
  } else {
    stop("Argument include_x_text_title must be TRUE or FALSE")
  }

  if(include_y_text_title == T){
    axis.title.y  = ggplot2::element_text(family = "sans", size = 7, color = 'black',margin = margin(t = 0, r = 4, b = 0, l = 0, unit = 'mm'))
  } else if(include_y_text_title == F){
    axis.title.y  = ggplot2::element_blank()
  } else {
    stop("Argument include_y_text_title must be TRUE or FALSE")
  }

  # Dados externos (legendas): Frutiger LT Pro 55 roman, tamanho 7, preto 100%
  if(legend.position == 'bottom'){
    # legend.box.spacing = unit(-1, "mm")
    # legend.box.margin=margin(r = -10,l = -10,b = -10,t = -10)
    # legend.text = ggplot2::element_text(margin = margin(r = 4, l = 1, b = 0, t = 0,  unit = 'mm'))
    legend.box.spacing = unit(0.2, "cm")  # Valor padrão para legend.box.spacing
    legend.text        = ggplot2::element_text(size = 7, color = "black")  # Valor padrão para legend.text
    legend.box.margin  = margin(0, 0, 0, 0)  # Valor padrão para legend.box.margin
  } else{
    legend.box.spacing = unit(0.2, "cm")  # Valor padrão para legend.box.spacing
    legend.text        = ggplot2::element_text(size = 7, color = "black")  # Valor padrão para legend.text
    legend.box.margin  = margin(0, 0, 0, 0)  # Valor padrão para legend.box.margin
  }
  legend.title = ggplot2::element_text(size = 7, color = "black")


  # Define the strip background and text styles for other box options
  strip.background = ggplot2::element_rect(fill = "white")
  strip.text       = ggplot2::element_text(colour = 'black',hjust=0)




  theme <- ggplot2::theme(
    text = ggplot2::element_text(family = "sans"),
    #text = ggplot2::element_text(family = "sans",size = unit(6, unit = 'mm')),
    # Sets the background color of the panel to white
    panel.background = ggplot2::element_rect(fill = "white", colour = NA),
    # Sets the panel border based on the previous assignment
    panel.border = panel.border,
    # Sets the major grid lines color and size
    #panel.grid.major = ggplot2::element_line(colour = color, linewidth = 0.25),
    # Sets the minor grid lines color and size
    panel.grid.minor = ggplot2::element_line(colour = "white", linewidth = 1),
    # Hides the major grid lines on the x-axis
    panel.grid.major.x = panel.grid.major.x,
    # Hides the major grid lines on the x-axis
    panel.grid.major.y = panel.grid.major.y,
    # Sets the position of the legend based on the previous assignment
    legend.position = legend.position,
    # Sets the appearance of the legend key
    legend.key = ggplot2::element_rect(colour = "transparent", fill = "white"),
    # Sets the background color of the facet strip based on the previous assignment
    strip.background = strip.background,
    # Sets the text color of the facet strip based on the previous assignment
    strip.text = strip.text,
    # Sets the appearance of the x-axis line based on the previous assignment
    axis.line.x = axis.line.x,
    # Sets the appearance of the y-axis line based on the previous assignment
    axis.line.y = axis.line.y,
    axis.line.y.right = ggplot2::element_line(colour = color, linewidth = lwd_05pt),
    # Sets the appearance of the axis text based on the previous assignment
    axis.text.x = axis.text.x,
    axis.text.y = axis.text.y,
    # Change title x
    axis.title.x = axis.title.x,
    axis.title.y = axis.title.y,
    # Sets the appearance of the axis ticks based on the previous assignment
    axis.ticks.x = axis.ticks.x,
    axis.ticks.y = axis.ticks.y,
    # Sets the appearance of the plot title
    plot.title = ggplot2::element_text(
      # FullName (Frutiger LT 47 Light Condensed). FamillyName (Frutiger LT 47 LightCn)
      family = "sans",
      hjust = 0,  lineheight = .5,
      margin = margin(0,0,1,0, unit = 'mm')
    ),
    # Sets the appearance of the plot subtitle
    plot.subtitle = ggplot2::element_text(
      # FullName (Frutiger LT Std 57 Condensed). FamillyName (Frutiger LT Std)
      family = "sans",
      face = "bold", hjust = 0,  lineheight = 1,
      margin = margin(0,0,2,0, unit = 'mm'),
    ),
    # axis.text = ggplot2::element_text(family = "sans", size = unit(6, "pt")),
    plot.margin=unit(c(.2,.5,.2,.2),"cm"),
    # Spacing between faceted plots
    panel.spacing = unit(4, "mm"),
    # Sets the appearance of the legend text
    #legend.text = ggplot2::element_text(size = 7),
    # Set caption position
    plot.caption = ggplot2::element_text(family = "sans", hjust = 0, vjust = 0,  lineheight = 1.25),
    # Set the horizontal alignment of the legend to center
    legend.justification = "center",
    # Set legend spacing y
    # Set horizontal and vertical spacing between legend keys (2 right,2 bottom,1 left)
    #legend.spacing = margin(t = 1,4,4,1, unit = 'mm'),
    # Set Margin spacing
    legend.box.spacing = legend.box.spacing,
    #legend.margin  = margin(t = 0,r = 15, b= 0,l = 1, unit = 'mm'),
    #change legend key size
    # legend.key.size = unit(1, 'mm'),
    legend.box.margin =  legend.box.margin,
    # Set key size
    # legend.spacing.x = unit(4, 'mm'),
    # legend.spacing.y = unit(4, 'mm'),
    legend.text = legend.text,
    legend.title = legend.title,
    legend.key.size = unit(4,"mm","linewidth"),
    # Adjust haste length
    axis.ticks.length = unit(2, "mm"),
    ...
  )

  # Verificação do tipo de dados para x_var no plot
  x_var <- plot$mapping$x
  x_var <- rlang::eval_tidy(x_var, plot$data)

  # Verificação do tipo de dados para y_var no plot
  y_var <- plot$mapping$y
  y_var <- rlang::eval_tidy(y_var, plot$data)

  for (i in seq_along(plot$layers)) {
    if ("x" %in% names(plot$layers[[i]]$mapping) & !is.null(nrow(plot$layers[[i]]$data))) {

      x_var_temp <- plot$layers[[i]]$mapping$x
      x_var_temp <- rlang::eval_tidy(x_var_temp, plot$layers[[i]]$data)
      break

    } else if("x" %in% names(plot$layers[[i]]$mapping) & is.null(nrow(plot$layers[[i]]$data))){

      x_var_temp <- plot$layers[[i]]$mapping$x
      x_var_temp <- rlang::eval_tidy(x_var_temp, plot$data)
      break

    } else {
      x_var_temp <- plot$mapping$x
      x_var_temp <- rlang::eval_tidy(x_var_temp, plot$data)
    }

  }


  for (i in seq_along(plot$layers)) {
    if ("y" %in% names(plot$layers[[i]]$mapping) & !is.null(nrow(plot$layers[[i]]$data))) {

      y_var_temp <- plot$layers[[i]]$mapping$y
      y_var_temp <- rlang::eval_tidy(y_var_temp, plot$layers[[i]]$data)
      break

    } else if("y" %in% names(plot$layers[[i]]$mapping) & is.null(nrow(plot$layers[[i]]$data))){

      y_var_temp <- plot$layers[[i]]$mapping$y
      y_var_temp <- rlang::eval_tidy(y_var_temp, plot$data)
      break

    } else {
      y_var_temp <- plot$mapping$y
      y_var_temp <- rlang::eval_tidy(y_var_temp, plot$data)
    }

  }

  if(is.null(x_var)){
    x_var <- x_var_temp
  } else{
    x_var <- x_var
  }

  if(is.null(y_var)){
    y_var <- y_var_temp
  } else{
    y_var <- y_var
  }



  test <- NULL
  for (i in seq_along(plot$layers)) {
    temp <- ifelse(grepl('bar',plot$layers[[i]]$constructor),'Bar chart',
                   ifelse(grepl('ribbon',plot$layers[[i]]$constructor),'Ribbon',
                          ifelse(grepl('area',plot$layers[[i]]$constructor),'Area',
                                 ifelse(grepl('histogram',plot$layers[[i]]$constructor),'Histogram',
                                        ifelse(grepl('density',plot$layers[[i]]$constructor),'Density',NA)))))
    temp <- temp[!is.na(temp)]
    test <- rbind(test,temp)


  }

  test <- unique(test)

  if(length(test) == 1){
    if(grepl('Bar',test)){

    } else {
      test = paste0(test,' graph')
    }

  } else if(length(test) > 1){
    test[length(test)] <- paste0('and ',test[length(test)],' graph')

    if(grepl('Bar',test)){
      test <- gsub(' Chart','',test)
    }

  } else {
    test <- NULL
  }


  if(expand_y_limit == TRUE & length(test) > 0){
    message(gsub(", and "," and ",paste0('Warning message:\n  Due to the existence of a ',paste(test, collapse = ", "),', the `expand_y_limit` argument will be converted to FALSE')))
    expand_y_limit = FALSE
  }

  if(!is.null(x_breaks) & length(test[grepl('Bar',test)]) > 0){
    message(gsub(", and "," and ",paste0('Warning message:\n  Due to the existence of a Bar chart the `x_breaks` argument will be converted to the number of options available')))
    x_breaks = length(unique(x_var))
  }

  if(!is.null(x_breaks) & length(test) > 0 & length(test[grepl('Bar',test)]) == 0 & length(unique(x_var)) <= 20){
    message(gsub(", and "," and ",paste0('Warning message:\n  Due to the existence of a ',paste(test, collapse = ", "),'and the number of values on the x-axis is small the `x_breaks` argument will be converted to the number of options available')))
    x_breaks = length(unique(x_var))
  }

  # Aplicação condicional das escalas
  # Escala para x_var
  if (!is.null(x_breaks) && is.numeric(x_var)) {
    plot <- plot + scale_x_continuous(
      breaks = my_pretty_breaks(x_var,x_breaks),
      expand = expansion(mult = c(ifelse(expand_x_limit == TRUE, 0.03, 0),
                                  ifelse(expand_x_limit == TRUE, 0.03, 0))),
      labels = scales::label_comma(decimal.mark = ",", big.mark = "")
    )
  } else if(is.null(x_breaks) && is.numeric(x_var)) {
    plot <- plot + scale_x_continuous(
      #breaks = my_pretty_breaks(x_var),
      expand = expansion(mult = c(ifelse(expand_x_limit == TRUE, 0.03, 0),
                                  ifelse(expand_x_limit == TRUE, 0.03, 0))),
      labels = scales::label_comma(decimal.mark = ",", big.mark = "")
    )
  } else {

  }


  # Escala para y_var
  if (!is.null(y_breaks) && is.numeric(y_var)) {
    plot <- plot + scale_y_continuous(
      breaks = my_pretty_breaks(y_var,y_breaks),
      expand = expansion(mult = c(ifelse(expand_y_limit == TRUE, 0.03, 0),
                                  ifelse(expand_y_limit == TRUE, 0.03, 0))),
      labels = scales::label_comma(decimal.mark = ",", big.mark = "")
    )
  } else if(is.null(y_breaks) && is.numeric(y_var)) {
    plot <- plot + scale_y_continuous(
      #breaks = my_pretty_breaks(y_var),
      expand = expansion(mult = c(ifelse(expand_y_limit == TRUE, 0.03, 0),
                                  ifelse(expand_y_limit == TRUE, 0.03, 0))),
      labels = scales::label_comma(decimal.mark = ",", big.mark = "")
    )
  } else {

  }

  # Marcadores quadrados: o Manual Editorial do Ipea usa marcador quadrado
  # solido nos pontos de graficos de linha (ver referencia no topo do
  # arquivo). Só aplicado quando o plot tem uma camada de linha (geom_line/
  # geom_path/geom_step) junto com a de pontos, para não afetar graficos de
  # dispersao (scatter) puros. Não sobrescreve um shape ja definido (fixo ou
  # mapeado) pelo usuario.
  layer_fn_name <- function(layer) {
    call_str <- paste(deparse(layer$constructor), collapse = " ")
    fn_name <- sub("\\s*\\(.*$", "", trimws(call_str))
    sub("^.*::", "", fn_name) # remove um eventual prefixo de namespace (ex.: "ggplot2::geom_point")
  }
  layer_fns <- vapply(plot$layers, layer_fn_name, character(1))
  has_line_layer <- any(layer_fns %in% c("geom_line", "geom_path", "geom_step"))

  if (has_line_layer) {
    for (i in seq_along(plot$layers)) {
      is_point_layer <- layer_fns[i] == "geom_point"
      shape_already_set <- "shape" %in% names(c(plot$mapping, plot$layers[[i]]$mapping)) ||
        !is.null(plot$layers[[i]]$aes_params$shape)

      if (is_point_layer && !shape_already_set) {
        plot$layers[[i]]$aes_params$shape <- 15
      }
    }
  }

  plot <- plot + theme



  return(plot)
}

base_plot <- function() {
  ggplot2::ggplot(mtcars, ggplot2::aes(hp, mpg)) + ggplot2::geom_point()
}

test_that("theme_ipea() defaults legend.position to bottom", {
  p <- base_plot() + theme_ipea()
  expect_equal(p$theme$legend.position, "bottom")
})

test_that("theme_ipea() validates axis_lines, grid.adjust and legend.position", {
  expect_error(base_plot() + theme_ipea(axis_lines = "bad"))
  expect_error(base_plot() + theme_ipea(grid.adjust = "bad"))
  expect_error(base_plot() + theme_ipea(legend.position = "bad"))
})

test_that("theme_ipea() validates axis_values", {
  expect_error(base_plot() + theme_ipea(axis_values = "bad"), "axis_values must be TRUE or FALSE")
})

test_that("theme_ipea() gives geom_point() a square marker only alongside a line layer", {
  p_scatter <- base_plot() + theme_ipea()
  expect_null(p_scatter$layers[[1]]$aes_params$shape)

  p_line <- ggplot2::ggplot(mtcars, ggplot2::aes(hp, mpg)) +
    ggplot2::geom_line() +
    ggplot2::geom_point() +
    theme_ipea()
  expect_equal(p_line$layers[[2]]$aes_params$shape, 15)
})

test_that("theme_ipea() never overrides a shape already set by the user", {
  p <- ggplot2::ggplot(mtcars, ggplot2::aes(hp, mpg)) +
    ggplot2::geom_line() +
    ggplot2::geom_point(shape = 17) +
    theme_ipea()
  expect_equal(p$layers[[2]]$aes_params$shape, 17)
})

test_that("theme_ipea() x-axis labels don't collide when line count is mixed", {
  # Regression test: a stray vjust = -1 on axis.text.x misaligned single-line
  # category labels whenever other categories on the same axis wrapped to two
  # lines via "\n", making the single-line label overlap the axis title.
  df <- data.frame(g = factor(c("A\nB", "C", "D\nE"), levels = c("A\nB", "C", "D\nE")), v = c(3, 10, 5))
  p <- ggplot2::ggplot(df, ggplot2::aes(g, v)) + ggplot2::geom_col() + theme_ipea()
  expect_equal(p$theme$axis.text.x$vjust, 1)
})

library(ipeaplot)
library(ggplot2)

data(mtcars)

# escala continua default do ggplot
ggplot() +
  geom_point(data = mtcars, aes(x = hp, y = mpg, color = mpg)) +
  ipeaplot::theme_ipea(angle = 45)



ggplot() +
  geom_col(data = mtcars, aes(x = hp, y = mpg, color = mpg)) +
  ipeaplot::theme_ipea(text = T)


# escala continua do ipeaplot
ggplot() +
  geom_point(data = mtcars, aes(x = hp, y = mpg, color = mpg)) +
  ipeaplot::scale_color_ipea(discrete = F) +
  ipeaplot::theme_ipea()




##### Coverage ------------------------
library(covr)
library(testthat)
Sys.setenv(NOT_CRAN = "true")


# each function separately
a <- covr::function_coverage(fun=r5r::travel_time_matrix, test_file("tests/testthat/test-travel_time_matrix.R"))


# the whole package
Sys.setenv(NOT_CRAN = "true")
ipeaplot_cov <- covr::package_coverage(path = ".", type = "tests")
ipeaplot_cov

covr::report()








# checks spelling
library(spelling)
devtools::spell_check(pkg = ".", vignettes = TRUE, use_wordlist = TRUE)

# Update documentation
devtools::document(pkg = ".")




# find unicode errors
tools::showNonASCIIfile(file = './man/accessibility.Rd')


functions <- list.files(path = './R', all.files = T, recursive = T, full.names = T)
lapply(X=functions, FUN = tools::showNonASCIIfile)

docs <- list.files(path = './man', all.files = T, recursive = T, full.names = T)
lapply(X=docs, FUN = tools::showNonASCIIfile)


### CMD Check ----------------
# Check package errors
library(tictoc)
library(beepr)

devtools::document()

# LOCAL
tictoc::tic()
Sys.setenv(NOT_CRAN = "true")
devtools::check(pkg = ".",  cran = FALSE, env_vars = c(NOT_CRAN = "true"))
tictoc::toc()
beepr::beep()



# CRAN
tictoc::tic()
Sys.setenv(NOT_CRAN = "false")
devtools::check(pkg = ".",  cran = TRUE, env_vars = c(NOT_CRAN = "false"))
tictoc::toc()


devtools::check_win_release(pkg = ".")

# devtools::check_win_oldrelease()
# devtools::check_win_devel()


beepr::beep()



tictoc::tic()
devtools::check(pkg = ".",  cran = TRUE, env_vars = c(NOT_CRAN = "false"))
tictoc::toc()



# submit to CRAN -----------------
usethis::use_cran_comments('teste 2222, , asdadsad')

urlchecker::url_check()
devtools::check(remote = TRUE, manual = TRUE)
devtools::check_win_release()
devtools::check_mac_release()
rhub::check_for_cran(show_status = FALSE)


devtools::submit_cran()



### pkgdown: update website ----------------

# Run to build the website
pkgdown::build_site()

#' @title Scale color IPEA
#'
#' @description Generate a color palette (continuous or discrete) following the
#'              editorial guidelines used by the Institute for Applied Economic
#'              Research - Ipea.
#' @param palette A character vector specifying the available palette for the
#'        color palette. The default palette are `"Blue"`, but we can also change
#'        to `'Green'`, `'Orange'`, `'Pink'`, `'Pink-Deep'`, `'Red-Blue'`, `'Orange-Blue'`, `'Green-Blue'`,
#'        `'Red-Blue-White'`, `'Orange-Blue-White'`, `'Green-Blue-White'`,
#'        `'Viridis'`, `'Inferno'`, `'Magma'`, `'Plasma'`, `'Cividis'`, `'NT'`, `'TD'`.
#'        `'NT'` and `'TD'` automatically switch from their sequential (continuous)
#'        colours to a 12-colour qualitative set when the mapped variable is
#'        discrete -- there is no need to request `'NT-Categorical'`/`'TD-Categorical'`
#'        directly, though those names remain available for `ipea_pal()`/`ipea_palette()`.
#' @param palette_direction A logical argument specifying if the ordering of the colors
#'        will follow the default of the palette (when the argument is 1) or if it will
#'        have an inverted ordering (for cases where it is -1).
#' @param decimal.mark The character to be used to indicate the numeric decimal point and
#'        Character used between every 3 digits to separate thousands.
#'        By default, the function uses a comma `","`, following
#'        the format used in Brazilian Portuguese.
#' @param barheight The height of the color gradient bar. This parameter is used
#'        when the direction is set to "horizontal".
#' @param barwidth The width of the color gradient bar. This parameter is used
#'        when the direction is set to "horizontal".
#' @param title.hjust A number specifying horizontal justification of the title text.
#' @param label.hjust A number specifying vertical justification of the title text.
#' @param ... Additional arguments to be passed to the scale_fill_gradientn,
#'        scale_color_gradientn, scale_fill_distiller or scale_color_distiller
#'        function from the ggplot2 package.
#'
#' @return A list object be added to a ggplot object to change color pallete.
#'
#' @import ggplot2 ggthemes rlang
#' @export
#' @family ggplot2 theme functions
#'
#' @examples
#' # Creating scale for ggplot2 graph using default arguments
#' library(ggplot2)
#' fig_raw <- ggplot() +
#'   geom_point(data = mtcars, aes(x = hp , y = mpg, color = cyl)) +
#'   scale_color_ipea()
#'
#' # Creating scale for ggplot2 graph using green sequential palette
#' fig_raw <- ggplot() +
#'   geom_point(data = mtcars, aes(x = hp , y = mpg, color = cyl)) +
#'   scale_color_ipea(palette = "Green")
#'
#' # "NT" uses the sequential magenta ramp for a numeric variable, and the
#' # 12-colour qualitative set for a discrete one, automatically
#' fig_raw <- ggplot() +
#'   geom_point(data = mtcars, aes(x = hp , y = mpg, color = factor(cyl))) +
#'   scale_color_ipea(palette = "NT")

# Definition of the scale_color_ipea function
scale_color_ipea <- function(palette = c('Blue','Green','Orange','Pink','Pink-Deep','Green-Blue','Green-Blue-White','Red-Blue','Red-Blue-White',
                                         'Orange-Blue','Orange-Blue-White', 'Viridis','Inferno','Magma','Plasma','Cividis',
                                         'NT','NT-Categorical','TD','TD-Categorical'),
                             palette_direction = 1,
                             decimal.mark = ",",
                             barheight = NULL, barwidth = NULL,
                             title.hjust = NULL, label.hjust = NULL,
                             ...) {

  structure(list(palette = palette, palette_direction = palette_direction,
                 decimal.mark = decimal.mark, barheight = barheight, barwidth = barwidth,
                 title.hjust = title.hjust, label.hjust = label.hjust,...), class = "scale_ipea_color")
}

#' @export
#' @method ggplot_add scale_ipea_color
# Definition of the ggplot_add.scale_ipea_color method
ggplot_add.scale_ipea_color <- function(object, plot, object_name, ...){

  # Set arguments
  args <- object
  palette <- args$palette
  palette_direction <- args$palette_direction
  decimal.mark <- args$decimal.mark
  barheight <- args$barheight
  barwidth <- args$barwidth
  title.hjust <- args$title.hjust
  label.hjust <- args$label.hjust

  # Set palette to 'ipea1' if it is not provided, otherwise use the provided value
  palette <-  ifelse(missing(palette),'Blue',palette)


  if (decimal.mark == ",") {
    # Use comma as decimal mark and dot as thousand separator for labels (Brazilian Portuguese)
    labels = scales::label_comma(decimal.mark = ",", big.mark = ".")
  } else if (decimal.mark == "."){
    # Use dot as decimal mark and comma as thousand separator for labels (default)
    labels = scales::label_comma(decimal.mark = ".", big.mark = ",")
  } else {
    stop("Decimal.mark argument must be '.' or ','.")
  }

  # Set default arguments
  label.hjust = ifelse(is.null(label.hjust),0.5,label.hjust)
  title.hjust = ifelse(is.null(title.hjust),0.5,title.hjust)

  if(is.null(barheight)){
    barheight = NULL
  } else{
    barheight = unit(barheight, units = "mm")
  }

  if(is.null(barwidth)){
    barwidth = NULL
  } else{
    barwidth = unit(barwidth, units = "mm")
  }



  # Find the first layer with the defined 'color' aesthetic
  # If ggplot includes aesthetics within geom_* this will recognize the class of the argument based on what is inside it
  for (i in seq_along(plot$layers)) {
    if ("colour" %in% names(plot$layers[[i]]$mapping)) {
      colour_var <- plot$layers[[i]]$mapping$colour
      var_evaluated <- rlang::eval_tidy(colour_var, plot$layers[[i]]$data)
      break
    } else {
      colour_var <- NULL
    }
  }


  # If 'colour_var' was not found in any layer, check in the base ggplot call
  if (is.null(colour_var)) {
    colour_var <- plot$mapping$colour
    var_evaluated <- rlang::eval_tidy(colour_var, plot$data)
  }

  # If 'colour_var' is not yet found, returns the graphic without changes
  if (is.null(colour_var)) {
    return(plot)
  }

  # Evaluates whether the variable is numeric.
  # Based on this criterion, we will select whether the palette will be discrete or continuous.
  auto_discrete_choose <- !is.numeric(var_evaluated)

  # 'NT' and 'TD' have a dedicated qualitative variant ('NT-Categorical',
  # 'TD-Categorical'); switch to it automatically for discrete variables so
  # users don't need to know the suffix exists.
  if (auto_discrete_choose && palette %in% c("NT", "TD")) {
    palette <- paste0(palette, "-Categorical")
  }

  # Choose the correct scale
  if (auto_discrete_choose) {

    return(plot +
             list(ggplot2::discrete_scale(aesthetics = "color", scale_name = "ipea", ipea_pal(palette = palette, palette_direction = palette_direction), ...),
                  ggplot2::guides(colour = guide_legend(...))))
  } else {
    scale_manual_pal <- ipea_palette(palette = palette, n = 10, palette_direction = palette_direction)
    return(
      plot +
        list(ggplot2::scale_color_gradientn(
          labels = labels,  # Set the labels for the gradient scale
          colours = scale_manual_pal,  # Set the scale_manual_pal for the gradient scale
          ...),
          guides(color =   guide_coloursteps(
            label.hjust = label.hjust,
            title.hjust = title.hjust,
            barheight = barheight,
            barwidth = barwidth,
            title.position = 'top',
            even.steps = F,...))))
  }
}

#' @title Scale fill IPEA
#'
#' @description Generate a fill palette (continuous or discrete) in the
#'              formatting of texts published by the Institute for Applied
#'              Economic Research (IPEA)
#' @param palette A character vector specifying the available palette for the
#'        color palette. The default palette are `"Blue"`, but we can also change
#'        to `'Green'`, `'Orange'`, `'Pink'`, `'Pink-Deep'`, `'Red-Blue'`, `'Orange-Blue'`, `'Green-Blue'`,
#'        `'Red-Blue-White'`, `'Orange-Blue-White'`, `'Green-Blue-White'`,
#'        `'Viridis'`, `'Inferno'`, `'Magma'`, `'Plasma'`, `'Cividis'`, `'NT'`, `'TD'`.
#'        `'NT'` and `'TD'` automatically switch from their sequential (continuous)
#'        colours to a 12-colour qualitative set when the mapped variable is
#'        discrete -- there is no need to request `'NT-Categorical'`/`'TD-Categorical'`
#'        directly, though those names remain available for `ipea_pal()`/`ipea_palette()`.
#' @param palette_direction A logical argument specifying if the ordering of the colors
#'        will follow the default of the palette (when the argument is 1) or if it will
#'        have an inverted ordering (for cases where it is 0).
#' @param decimal.mark The character to be used to indicate the numeric decimal point and
#'        Character used between every 3 digits to separate thousands.
#'        By default, the function uses a comma `","`, following
#'        the format used in Brazilian Portuguese.
#' @param barheight The height of the color gradient bar. This parameter is used
#'        when the direction is set to "horizontal".
#' @param barwidth The width of the color gradient bar. This parameter is used
#'        when the direction is set to "horizontal".
#' @param title.hjust A number specifying horizontal justification of the title text.
#' @param label.hjust A number specifying vertical justification of the title text.
#' @param ... Additional arguments to be passed to the scale_fill_gradientn,
#'        scale_color_gradientn, scale_fill_distiller or scale_color_distiller
#'        function from the ggplot2 package
#'
#'
#' @return A list object be added to a ggplot object to change color pallete.
#'
#' @export
#' @family ggplot2 theme functions
#'
#' @examples
#' # Creating scale for ggplot2 graph using default arguments
#' library(ggplot2)
#' fig_raw <- ggplot() +
#'   geom_col(data = mtcars, aes(x = hp , y = mpg, fill = cyl)) +
#'   scale_fill_ipea()
#'
#' # Creating scale for ggplot2 graph using green sequential palette
#' fig_raw <- ggplot() +
#'   geom_col(data = mtcars, aes(x = hp , y = mpg, fill = cyl)) +
#'   scale_fill_ipea(palette = "Green")
#'
#' # "TD" uses the sequential blue ramp for a numeric variable, and the
#' # 12-colour qualitative set for a discrete one, automatically
#' fig_raw <- ggplot() +
#'   geom_col(data = mtcars, aes(x = hp , y = mpg, fill = factor(cyl))) +
#'   scale_fill_ipea(palette = "TD")
scale_fill_ipea <- function(palette = c('Blue','Green','Orange','Pink','Pink-Deep','Green-Blue','Green-Blue-White','Red-Blue','Red-Blue-White',
                                        'Orange-Blue','Orange-Blue-White', 'Viridis','Inferno','Magma','Plasma','Cividis',
                                        'NT','NT-Categorical','TD','TD-Categorical'),
                            palette_direction = 1,
                            decimal.mark = ",",
                            barheight = NULL, barwidth = NULL,
                            title.hjust = NULL, label.hjust = NULL,
                            ...) {

  structure(list(palette = palette, palette_direction = palette_direction,
                 decimal.mark = decimal.mark, barheight = barheight, barwidth = barwidth,
                 title.hjust = title.hjust, label.hjust = label.hjust,...), class = "scale_ipea_fill")
}

#' @export
#' @method ggplot_add scale_ipea_fill
# Definition of the ggplot_add.scale_ipea_fill method
ggplot_add.scale_ipea_fill <- function(object, plot, object_name, ...){

  # Set arguments
  args <- object
  palette <- args$palette
  palette_direction <- args$palette_direction
  decimal.mark <- args$decimal.mark
  barheight <- args$barheight
  barwidth <- args$barwidth
  title.hjust <- args$title.hjust
  label.hjust <- args$label.hjust

  # Set palette to 'ipea1' if it is not provided, otherwise use the provided value
  palette <-  ifelse(missing(palette),'Blue',palette)


  if (decimal.mark == ",") {
    # Use comma as decimal mark and dot as thousand separator for labels (Brazilian Portuguese)
    labels = scales::label_comma(decimal.mark = ",", big.mark = ".")
  } else if (decimal.mark == "."){
    # Use dot as decimal mark and comma as thousand separator for labels (default)
    labels = scales::label_comma(decimal.mark = ".", big.mark = ",")
  } else {
    stop("Decimal.mark argument must be '.' or ','.")
  }

  # Set default arguments
  label.hjust = ifelse(is.null(label.hjust),0.5,label.hjust)
  title.hjust = ifelse(is.null(title.hjust),0.5,title.hjust)

  if(is.null(barheight)){
    barheight = NULL
  } else{
    barheight = unit(barheight, units = "mm")
  }

  if(is.null(barwidth)){
    barwidth = NULL
  } else{
    barwidth = unit(barwidth, units = "mm")
  }



  # Find the first layer with the defined 'fill' aesthetic
  # If ggplot includes aesthetics within geom_* this will recognize the class of the argument based on what is inside it
  for (i in seq_along(plot$layers)) {
    if ("fill" %in% names(plot$layers[[i]]$mapping)) {
      fill_var <- plot$layers[[i]]$mapping$fill
      var_evaluated <- rlang::eval_tidy(fill_var, plot$layers[[i]]$data)
      break
    } else {
      fill_var <- NULL
    }
  }


  # If 'fill_var' was not found in any layer, check in the base ggplot call
  if (is.null(fill_var) && !is.null(plot$mapping$fill)) {
    fill_var <- plot$mapping$fill
    var_evaluated <- rlang::eval_tidy(fill_var, plot$data)
  }

  # If 'fill_var' is not yet found, returns the graphic without changes
  if (is.null(fill_var)) {
    return(plot)
  }

  # Evaluates whether the variable is numeric.
  # Based on this criterion, we will select whether the palette will be discrete or continuous.
  auto_discrete_choose <- !is.numeric(var_evaluated)

  # 'NT' and 'TD' have a dedicated qualitative variant ('NT-Categorical',
  # 'TD-Categorical'); switch to it automatically for discrete variables so
  # users don't need to know the suffix exists.
  if (auto_discrete_choose && palette %in% c("NT", "TD")) {
    palette <- paste0(palette, "-Categorical")
  }

  # Choose the correct scale
  if (auto_discrete_choose) {

    return(plot +
             list(ggplot2::discrete_scale(aesthetics = "fill", scale_name = "ipea", ipea_pal(palette = palette, palette_direction = palette_direction), ...),
                  ggplot2::guides(fill = guide_legend(...))))
  } else {
    scale_manual_pal <- ipea_palette(palette = palette, n = 10, palette_direction = palette_direction)
    return(
      plot +
        list(ggplot2::scale_fill_gradientn(
          labels = labels,  # Set the labels for the gradient scale
          colours = scale_manual_pal,  # Set the scale_manual_pal for the gradient scale
          ...),
          guides(fill =   guide_coloursteps(
            label.hjust = label.hjust,
            title.hjust = title.hjust,
            barheight = barheight,
            barwidth = barwidth,
            title.position = 'top',
            even.steps = F,...))))
  }
}

test_that("scale_color_ipea() switches NT/TD to their categorical variant for discrete data", {
  p <- suppressWarnings(
    ggplot2::ggplot(mtcars, ggplot2::aes(hp, mpg, color = factor(cyl))) +
      ggplot2::geom_point() +
      scale_color_ipea(palette = "NT")
  )
  b <- ggplot2::ggplot_build(p)
  expect_length(unique(b$data[[1]]$colour), 3)
})

test_that("scale_color_ipea() keeps NT/TD sequential for continuous data", {
  p <- ggplot2::ggplot(mtcars, ggplot2::aes(hp, mpg, color = cyl)) +
    ggplot2::geom_point() +
    scale_color_ipea(palette = "NT")
  expect_silent(ggplot2::ggplot_build(p))
})

test_that("scale_color_ipea() leaves the plot untouched when there's no colour aesthetic", {
  p <- ggplot2::ggplot(mtcars, ggplot2::aes(hp, mpg)) +
    ggplot2::geom_point() +
    scale_color_ipea()
  expect_s3_class(p, "ggplot")
})

test_that("scale_color_ipea() validates decimal.mark", {
  expect_error(
    ggplot2::ggplot(mtcars, ggplot2::aes(hp, mpg, color = cyl)) +
      ggplot2::geom_point() +
      scale_color_ipea(decimal.mark = "x"),
    "Decimal.mark"
  )
})

test_that("scale_fill_ipea() switches NT/TD to their categorical variant for discrete data", {
  p <- suppressWarnings(
    ggplot2::ggplot(mtcars, ggplot2::aes(factor(cyl), mpg, fill = factor(cyl))) +
      ggplot2::geom_col() +
      scale_fill_ipea(palette = "TD")
  )
  b <- ggplot2::ggplot_build(p)
  expect_length(unique(b$data[[1]]$fill), 3)
})

test_that("scale_fill_ipea() validates decimal.mark", {
  expect_error(
    ggplot2::ggplot(mtcars, ggplot2::aes(factor(cyl), mpg, fill = cyl)) +
      ggplot2::geom_col() +
      scale_fill_ipea(decimal.mark = "x"),
    "Decimal.mark"
  )
})

#' @title Contrasting text colour for labels drawn over a fill colour
#'
#' @description Given one or more fill/background colours, returns `"white"`
#' or `"black"`, whichever gives better contrast against that background.
#' Useful for value labels drawn inside bars or over points/lines, matching
#' the Ipea editorial guidelines for graphics, which alternate the colour of
#' data labels according to how dark the colour behind them is. See
#' <https://www.ipea.gov.br/manualeditorial/padroes/padroes-grafico-visuais/ilustracoes-pi.html>.
#'
#' @param background A character vector of colours (hex codes or R colour
#'        names) used as the fill/background behind the label.
#'
#' @return A character vector the same length as `background`, containing
#'         `"white"` or `"black"`.
#' @export
#' @family ggplot2 theme functions
#'
#' @examples
#' ipea_contrast_color(c("#29112F", "#D2A0C9"))
#'
#' library(ggplot2)
#' df <- data.frame(grupo = c("A", "B"), valor = c(22.6, 4.7),
#'                   cor = c("#29112F", "#D2A0C9"))
#' ggplot(df, aes(grupo, valor, fill = cor, label = valor)) +
#'   geom_col() +
#'   geom_text(aes(color = ipea_contrast_color(cor)), size = ipea_label_size(),
#'             fontface = "bold") +
#'   scale_fill_identity() +
#'   scale_color_identity() +
#'   theme_ipea()
ipea_contrast_color <- function(background) {
  # unname() evita que os rownames "red"/"green"/"blue" de col2rgb() vazem
  # como names() do vetor de saida (acontece via apply() quando length(background) == 1).
  rgb_mat <- unname(grDevices::col2rgb(background) / 255)

  # Luminancia relativa (WCAG): converte cada canal sRGB para linear antes de
  # ponderar pela sensibilidade do olho humano a cada cor.
  linear_channel <- function(ch) ifelse(ch <= 0.03928, ch / 12.92, ((ch + 0.055) / 1.055)^2.4)
  linear_rgb <- apply(rgb_mat, 2, linear_channel)
  luminance <- 0.2126 * linear_rgb[1, ] + 0.7152 * linear_rgb[2, ] + 0.0722 * linear_rgb[3, ]

  # 0.179 e o limiar em que o contraste com texto preto e branco se equivale.
  ifelse(luminance > 0.179, "black", "white")
}

test_that("ipea_contrast_color() picks white text for dark backgrounds", {
  expect_equal(ipea_contrast_color("#29112F"), "white")
  expect_equal(ipea_contrast_color("black"), "white")
})

test_that("ipea_contrast_color() picks black text for light backgrounds", {
  expect_equal(ipea_contrast_color("#D2A0C9"), "black")
  expect_equal(ipea_contrast_color("white"), "black")
})

test_that("ipea_contrast_color() is vectorised and preserves order", {
  out <- ipea_contrast_color(c("#29112F", "#D2A0C9", "white", "black"))
  expect_equal(out, c("white", "black", "black", "white"))
  expect_length(out, 4)
})

test_that("ipea_contrast_color() accepts a single colour", {
  expect_length(ipea_contrast_color("#29112F"), 1)
})

# Exemplo 00: Gráfico   -------------
remotes::install_github("ipeadata-lab/ipea_templates")


#library(ipeaplot)
library(ggplot2)
library(dplyr)
data("mtcars")

mtcars <- mtcars %>%
  mutate(am = ifelse(am == 1,"Manual","Automático"))


f <- function(k) {
  step <- k
  function(y) seq(floor(min(y)), ceiling(max(y)), by = step)
}



# continuous
ggplot() +
  geom_point(data=mtcars, aes(x=mpg , y=drat, color=drat)) +
  scale_color_ipea(discrete = F, palette = 'Reds') +
  facet_wrap(.~am) +
  labs(x="Weather    stations",
     y="Accumulated Rainfall [mm]",
     title="GRÁFICO 7",
     subtitle="Forma de organização do horário de trabalho",
     caption = 'Fonte: ipea') +
  theme_ipea(legend.position = 'right')


# discrete
ggplot(data=mtcars, aes(x=mpg , y=cyl, color=carb)) +
  geom_point() +
  scale_colour_discrete_ipea()


# Exemplo 01: Gráfico de pontos  -------------

cons = abjData::consumo

# Gráfico simples
ggplot(data=cons, aes(x=dec_date, y=tempo, color=assunto, group = assunto)) +
  geom_point() +
  xlab('Ano') +
  ylab('Tempo') +
  ggtitle(label = "Tempo", subtitle = "Brasil (%)") +
  labs(color = "Assunto") +
  ipea_style(legend.position = "right") +
  scale_color_discrete_ipea()




# Create a continuous scatter plot with 'mpg' on the x-axis, 'wt' on the y-axis, and color-coded by 'wt'
# Use the 'scale_ipea()' function to apply the IPEA continuous color scale
ggplot(data = mtcars, aes(x = mpg, y = wt, color = wt)) +
  geom_point() +
  theme_ipea(type = "continuous", style = "color")


# Create a continuous scatter plot with 'mpg' on the x-axis, 'wt' on the y-axis, and filled by 'wt'
# Use the 'scale_ipea()' function to apply the IPEA continuous fill scale
ggplot(data = mtcars, aes(x = mpg, y = wt, fill = wt)) +
  geom_bar(stat="identity", position=position_dodge()) +
  theme_ipea(type = "continuous", style = "fill")


# Create a discrete scatter plot with 'mpg' on the x-axis, 'wt' on the y-axis, and color-coded by 'quantile'
# Use the 'scale_ipea()' function to apply the IPEA discrete color scale
ggplot(data = mtcars, aes(x = mpg, y = wt, color = quantile)) +
  geom_point() +
  theme_ipea(type = "discrete", style = "color")



# Gráfico simples (sem legenda)
ggplot(data=cons, aes(x=dec_date, y=tempo, color=assunto, group = assunto)) +
  geom_point() +
  xlab('Ano') +
  ylab('Tempo') +
  ggtitle(label = "Tempo", subtitle = "Brasil (%)")+
  ipea_style(legend.position = "none") +
  scale_color_discrete_ipea()

# Exemplo 02: Gráfico de linha -------------
graph <- abjData::pnud_muni %>%
  group_by(ano) %>%
  dplyr::summarise(espvida = mean(espvida),
                   rdpc = mean(rdpc))

ggplot(data=graph, aes(x=as.character(ano), y=espvida, color = espvida, group = 1)) +
  geom_line() +
  geom_point() +
  xlab('Ano') +
  ylab('Expectativa de vida') +
  theme_ipea(type = 'continuous', style = 'color',
             label = 'espvida',
             legend.position = 'none') +
  ggtitle(label = "Expectativa de vida", subtitle = "Brasil (%)")

# Exemplo 03: Gráfico de barras  -------------

# Exemplo 3.1 - Gráfico padrão  -------------
graph <- abjData::pnud_muni %>%
  group_by(ano) %>%
  dplyr::summarise(espvida = mean(espvida),
                   rdpc = mean(rdpc))

ggplot(data=graph, aes(x=as.character(ano), y=espvida, fill=espvida)) +
  geom_bar(stat="identity") +
  labs(x="Ano",
       y="Expectativa de vida",
       fill = "Expectativa de vida",
       title="Gráfico 1",
       subtitle="Expectativa de vida",
       caption = 'Fonte: ipea') +
  insert_text(label = "espvida", decimals = 0) +
  theme_ipea() +
  scale_fill_ipea()

ggplot() +
  geom_point(data=mtcars, aes(x=mpg , y=cyl, color=carb)) +
  scale_color_ipea(discrete = F, palette = 'ipeatd') +
  facet_wrap(.~am) +

  theme_ipea(legend.position = 'right')

# Exemplo 3.1 - Gráfico para cada UF  -------------
graph <- abjData::pnud_muni %>%
  mutate(regn = substr(uf,1,1),
         regn = case_when(regn == "1"~"Norte",
                          regn == "2"~"Nordeste",
                          regn == "3"~"Sudeste",
                          regn == "4"~"Sul",
                          regn == "5"~"Centro Oeste", TRUE ~ regn)) %>%
  group_by(regn,ano) %>%
  dplyr::summarise(espvida = mean(espvida),
                   rdpc = mean(rdpc))

# Opção 1 de cor do título
ggplot(data=graph, aes(x=as.character(ano), y=espvida, fill=espvida)) +
  geom_bar(stat="identity") +
  xlab('Ano') +
  ylab('Expectativa de vida') +
  insert_text(label = "espvida", decimals = 0) +
  facet_wrap(.~regn) +
  scale_y_continuous(labels = scales::label_comma(decimal.mark = ",", big.mark = ".")) +
  ggtitle(label = "Expectativa de vida", subtitle = "Brasil (%)")+
  ipea_style(box_option = 0) +
  scale_fill_continuous_ipea() +
  coord_cartesian(ylim = c(min(graph$espvida),max(graph$espvida)+10))

# Opção 2 de cor do título
ggplot(data=graph, aes(x=as.character(ano), y=espvida, fill=espvida)) +
  geom_bar(stat="identity") +
  xlab('Ano') +
  ylab('Expectativa de vida') +
  insert_text(label = "espvida", decimals = 0) +
  facet_wrap(.~regn) +
  scale_y_continuous(labels = scales::label_comma(decimal.mark = ",", big.mark = ".")) +
  ggtitle(label = "Expectativa de vida", subtitle = "Brasil (%)")+
  ipea_style() +
  scale_fill_continuous_ipea() +
  coord_cartesian(ylim = c(min(graph$espvida),max(graph$espvida)+10))


# Exemplo 4 - Mapa do Brasil  -------------
mun = geobr::read_municipality()
uf  = geobr::read_state()
pnud = abjData::pnud_muni %>%
  subset(ano == 2010) %>%
  select(ano,code_muni = codmun7,e_anosestudo,rdpc)
df = left_join(mun, pnud)
a = subset(df,abbrev_state == "PB")

graf2 <- ggplot(data = a, aes(fill = e_anosestudo)) +
  geom_sf() +
  #geom_sf(data = uf, color = "black", fill = NA) +
  ggtitle("Anos de estudo", "Em anos") +
  xlab("")+ylab("") +
  ipea_style(axis = "full", text = F) +
  scale_fill_continuous_ipea( options = c("ipea1"))

graf2

# Caso deseja mudar a variação da escala
graf2 <- ggplot(data = df, aes(fill = e_anosestudo)) +
  geom_sf(color = NA) +
  geom_sf(data = uf, color = "black", fill = NA) +
  ggtitle("Anos de estudo", "Em anos") +
  xlab("")+ylab("") +
  theme_ipea(axis = "none", text = F,
             show.limits = T,name = "Anos de estudo",
             type = 'continuous', style = 'fill')
graf2

#' @title Save ggplot in multiple formats
#' @description Unified function to save ggplot objects in one or more formats.
#'
#' @param gplot ggplot object. Default: ggplot2::last_plot().
#' @param file.name Base name of the file (without extension). E.g., "results/figs/my_plot".
#' @param format Vector with one or more of: "eps", "jpg", "pdf", "png", "svg".
#'   If NULL, uses getOption("ipea.plot.default_format", "eps").
#' @param width,height Plot dimensions (default 160 x 100).
#' @param units Units: "mm", "cm", "in", "px" (default "in").
#' @param dpi DPI for raster formats (default 300).
#' @param background Background color; use NA for transparent PNGs.
#'                  For JPEG (no transparency), NA becomes "white".
#' @param quality JPEG quality (0 to 100) (default 95).
#' @param path Output directory (default ".").
#' @param scale Scale factor passed to ggsave (default 1).
#' @param overwrite If FALSE, does not overwrite existing files: creates suffixes _1, _2, ... (default TRUE).
#' @param include_date If TRUE, appends the date to the file name: _YYYYMMDD (default FALSE).
#' @param date_format Date format (default "%Y%m%d").
#' @param use_cairo Prefer using Cairo for PDF/EPS (default TRUE).
#' @param use_ragg Prefer using ragg for PNG/JPG when available (default TRUE).
#' @param quiet Suppress messages from ggsave when supported (default TRUE).
#' @param ... Passed to ggplot2::ggsave() (and therefore to the underlying graphics devices when applicable).
#'
#' @return Named (invisible) vector with the paths of the saved files.
#' @import ragg
#' @export
save_ipeaplot <- function(
    gplot = ggplot2::last_plot(),
    file.name,
    format = NULL,
    width = 6.30,
    height = 3.94,
    units = c("in", "mm", "cm", "px"),
    dpi = 300,
    background = "white",
    quality = 95,
    path = ".",
    scale = 1,
    overwrite = TRUE,
    include_date = FALSE,
    date_format = "%Y%m%d",
    use_cairo = TRUE,
    use_ragg = TRUE,
    quiet = TRUE,
    ...
) {
  if (!inherits(gplot, "ggplot"))
    stop("gplot must be a ggplot2 object.", call. = FALSE)
  if (
    !is.character(file.name) ||
    length(file.name) != 1L ||
    nchar(file.name) == 0L
  )
    stop("file.name must be a non-empty string.", call. = FALSE)

  units <- match.arg(units)

  if (is.null(format)) {
    format <- getOption("ipea.plot.default_format", "eps")
  }

  valid <- c("eps", "jpg", "pdf", "png", "svg")
  format <- unique(match.arg(format, choices = valid, several.ok = TRUE))

  # Auto-conversao de "mm digitado como in" (ex.: width=160, height=100 com units="in")
  if (units == "in" && (width >= 50 || height >= 50)) {
    width <- width / 25.4
    height <- height / 25.4
    message(sprintf(
      "Dimensions too large in 'in'; interpreting as mm -> width=%.2f in, height=%.2f in.",
      width,
      height
    ))
  }

  base_name <- sub("\\.[^.]+$", "", file.name)
  if (include_date)
    base_name <- paste0(base_name, "_", format(Sys.Date(), date_format))

  ext <- function(fmt)
    switch(fmt, eps = ".eps", jpg = ".jpg", pdf = ".pdf", png = ".png", svg = ".svg")

  get_device <- function(fmt) {
    if (
      fmt %in%
      c("png", "jpg") &&
      use_ragg &&
      requireNamespace("ragg", quietly = TRUE)
    ) {
      return(if (fmt == "png") ragg::agg_png else ragg::agg_jpeg)
    }
    if (fmt == "png") return("png")
    if (fmt == "jpg") return("jpeg")
    if (fmt == "pdf")
      return(if (use_cairo) grDevices::cairo_pdf else grDevices::pdf)
    if (fmt == "eps")
      return(if (use_cairo) grDevices::cairo_ps else grDevices::postscript)
    if (fmt == "svg") {
      # Usa svglite se disponível; senão cai para o device base (grDevices::svg via "svg")
      if (requireNamespace("svglite", quietly = TRUE)) return(svglite::svglite)
      return("svg")
    }
  }

  dir.create(path, recursive = TRUE, showWarnings = FALSE)

  next_free_name <- function(fp) {
    if (!file.exists(fp)) return(fp)
    root <- sub("(\\.[^.]+)$", "", fp)
    ex <- sub("^.*(\\.[^.]+)$", "\\1", fp)
    i <- 1L
    repeat {
      candidate <- paste0(root, "_", i, ex)
      if (!file.exists(candidate)) return(candidate)
      i <- i + 1L
    }
  }

  ggsave_has_quiet <- "quiet" %in% names(formals(ggplot2::ggsave))

  out <- character(length(format))
  names(out) <- format

  for (i in seq_along(format)) {
    fmt <- format[i]
    file_path <- file.path(path, paste0(base_name, ext(fmt)))
    if (!overwrite) file_path <- next_free_name(file_path)

    dev <- get_device(fmt)

    # JPEG nao tem transparencia: NA -> "white"
    bg_use <- if (fmt == "jpg" && is.na(background)) "white" else background

    gargs <- list(
      filename = file_path,
      plot = gplot,
      device = dev,
      width = width,
      height = height,
      units = units,
      scale = scale
    )

    # Raster
    if (fmt %in% c("png", "jpg")) {
      gargs$dpi <- dpi
      gargs$bg <- bg_use
    }

    # Vetoriais: alguns argumentos so existem em determinados dispositivos, e
    # do.call() falha com "unused argument" se forem passados a um dispositivo
    # que nao os declara em seus formals(). Por isso cada um e adicionado
    # apenas quando o dispositivo realmente o aceita:
    # - useDingbats: apenas grDevices::pdf() (evita fontes dingbat)
    # - fallback_resolution: apenas dispositivos baseados em Cairo (cairo_pdf, cairo_ps)
    if (fmt %in% c("pdf", "eps", "svg") && is.function(dev)) {
      dev_formals <- names(formals(dev))
      if ("useDingbats" %in% dev_formals) {
        gargs$useDingbats <- FALSE
      }
      if ("fallback_resolution" %in% dev_formals) {
        gargs$fallback_resolution <- dpi
      }
      # grDevices::postscript() (fallback de "eps" quando use_cairo = FALSE),
      # ao contrario de pdf()/cairo_pdf()/cairo_ps(), so reconhece uma familia
      # de fonte por arquivo: a que for declarada no argumento `family` do
      # dispositivo. Como theme_ipea() sempre desenha o texto com
      # family = "sans", sem isso o ggsave() falha com
      # "family 'sans' not included in postscript() device".
      if (identical(dev, grDevices::postscript)) {
        gargs$family <- "sans"
      }
    }

    # JPEG: qualidade
    if (fmt == "jpg") {
      gargs$quality <- quality
    }

    gargs <- gargs[!vapply(gargs, is.null, logical(1))]

    dotlist <- list(...)
    if (ggsave_has_quiet) dotlist$quiet <- quiet
    if (length(dotlist)) gargs[names(dotlist)] <- dotlist

    do.call(ggplot2::ggsave, gargs)

    # Em sistemas sem as bibliotecas Cairo/X11, os dispositivos cairo_pdf()/
    # cairo_ps() podem falhar silenciosamente em C (sem lançar erro em R),
    # de modo que ggsave() "termina" sem nunca ter gravado o arquivo. Sem essa
    # checagem, save_ipeaplot() reportaria sucesso mesmo sem ter salvo nada.
    if (!file.exists(file_path) || file.size(file_path) == 0) {
      hint <- if (isTRUE(use_cairo) && fmt %in% c("pdf", "eps")) {
        paste(
          " O dispositivo Cairo pode ter falhado silenciosamente",
          "(bibliotecas Cairo/X11 ausentes no sistema); tente novamente",
          "com use_cairo = FALSE."
        )
      } else {
        ""
      }
      stop(
        sprintf(
          "Falha ao salvar '%s': o arquivo nao foi criado ou ficou vazio.%s",
          file_path, hint
        ),
        call. = FALSE
      )
    }

    out[i] <- file_path
  }

  invisible(out)
}

local_test_dir <- function() {
  td <- tempfile("ipeaplot-test-")
  dir.create(td)
  withr::defer(unlink(td, recursive = TRUE), envir = parent.frame())
  td
}

base_plot <- function() {
  ggplot2::ggplot(mtcars, ggplot2::aes(hp, mpg)) + ggplot2::geom_point()
}

test_that("save_ipeaplot() saves a png that exists and is non-empty", {
  td <- local_test_dir()
  out <- save_ipeaplot(base_plot(), file.name = "fig", format = "png", path = td)
  expect_true(file.exists(out))
  expect_gt(file.size(out), 0)
})

test_that("save_ipeaplot() saves multiple formats at once", {
  td <- local_test_dir()
  out <- save_ipeaplot(base_plot(), file.name = "fig", format = c("png", "pdf"),
                        path = td, use_cairo = FALSE)
  expect_length(out, 2)
  expect_true(all(file.exists(out)))
})

test_that("save_ipeaplot() creates numbered suffixes when overwrite = FALSE", {
  td <- local_test_dir()
  out1 <- save_ipeaplot(base_plot(), file.name = "fig", format = "png", path = td)
  out2 <- save_ipeaplot(base_plot(), file.name = "fig", format = "png", path = td, overwrite = FALSE)
  expect_false(out1 == out2)
  expect_match(out2, "fig_1\\.png$")
})

test_that("save_ipeaplot() appends the date when include_date = TRUE", {
  td <- local_test_dir()
  out <- save_ipeaplot(base_plot(), file.name = "fig", format = "png", path = td, include_date = TRUE)
  expect_match(out, format(Sys.Date(), "%Y%m%d"))
})

test_that("save_ipeaplot() validates its inputs", {
  td <- local_test_dir()
  expect_error(save_ipeaplot("not-a-ggplot", file.name = "x", format = "png", path = td),
               "gplot must be a ggplot2 object")
  expect_error(save_ipeaplot(base_plot(), file.name = "", format = "png", path = td),
               "file.name must be a non-empty string")
  expect_error(save_ipeaplot(base_plot(), file.name = "x", format = "docx", path = td))
})

#' ipeaplot: ggplot Graphics in Ipea Standard
#'
#' Convenient functions to create ggplot graphics following the editorial
#' guidelines of the  Institute for Applied Economic Research - Ipea.
#'
#' @section Usage:
#' Please check the vignettes and data documentation on the
#' [website](https://github.com/ipeadata-lab/ipeaplot).
#'
#' @docType package
#' @name ipeaplot
#' @aliases ipeaplot-package
#'
#' @importFrom grDevices postscript dev.off
"_PACKAGE"


## quiets concerns of R CMD check re: the .'s that appear in pipelines
utils::globalVariables( c('postscript',
                          'dev.off'
                          ) )

---
title: "Cookbook ipeaplot"
date: "`r Sys.Date()`"
output:
  html_document:
    df_print: paged
urlcolor: blue
Vignette: >
  %\VignetteIndexEntry{Cookbook} 
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}\n
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = identical(tolower(Sys.getenv("NOT_CRAN")), "true"),
  out.width = "100%"
)
```
O **ipeaplot** é um pacote em R desenvolvido para facilitar a padronização de gráficos e figuras seguindo a política editorial do Instituto de Pesquisa Econômica Aplicada (Ipea). 

Nesta vinheta, apresentamos exemplos práticos do uso do pacote para gerar visulizações de dados comuns em textos e relatórios do Ipea. 

Em algumas figuras, os ajustes necessários para atender às diretrizes estilísticas do Editorial do Ipea demandam a criação de códigos específicos, extrapolando as configurações pré-estabelecidas no **ipeaplot**. 

Essa vinheta tem, por isso, um formato de *cookbook*, proporcionando orientações práticas para a utilização de outros recursos do `ggplot2`.

Em todos os exemplos, é exibida a versão "bruta" da figura, tal como foi submetida à Coordenação do Editorial. Em seguida, indicamos como é possível, com o **ipeaplot** e algumas linhas de código adicionais, atingir o nível de padronização necessário às publicações do Ipea.

```{r eval=FALSE, message=FALSE, warning=FALSE, results='hide'}
# Leitura de bibliotecas
library(ipeaplot)
library(ggplot2)
library(dplyr)
library(data.table)
library(abjData)
library(reshape)
library(knitr)

```

```{r eval=FALSE, message=FALSE, warning=FALSE}
# Leitura dos dados
## Gráfico 1
Ano <- factor(c(2000,2005,2010,2015,2020,2025,2030,2035,2040,2045,2050))
Populacao <- c(6.1,6.5,6.9,7.3,7.7,8.0,8.3,8.6,8.9,9.1,9.3)
labels <- as.character(Populacao)
labels <- gsub("." , "," , labels, fixed=T)
base_graf1 <- data.frame(Ano,Populacao,labels)

## Gráfico 2
Ano <- factor(c(1950,	1951,	1952,	1953,	1954,	1955,	1956,	1957,	1958,	
                1959,	1960,	1961,	1962,	1963,	1964,	1965,	1966,	1967,
                1968,	1969,	1970,	1971,	1972,	1973,	1974,	1975,	1976,
                1977,	1978,	1979,	1980,	1981,	1982,	1983,	1984,	1985,
                1986,	1987,	1988,	1989,	1990,	1991,	1992,	1993,	1994,
                1995,	1996,	1997,	1998,	1999,	2000,	2001,	2002,	2003,
                2004,	2005,	2006,	2007,	2008,	2009,	2010,	2011,	2012,
                2013,	2014))
Populacao<- c( 1.850116458,	1.791417471,	1.760491225,	1.751060624,
               1.757374206,	1.773968217,	1.795950884,	1.81926587,
               1.840930397,	1.860403495,	1.879485845,	1.90178203,
               1.931384556,	1.96968284 ,	2.011263472,  2.052385316,
               2.083875031,	2.098137208,	2.091095193,	2.06831928,
               2.043373483,	2.018775478,	1.985792138,	1.944170268,
               1.898407101,	1.850171278,	1.807798778,	1.779963926,
               1.770930519,	1.77532254 ,	1.778741317,	1.77934085,	
               1.787498884,	1.803957627,	1.823106379,	1.844715108,
               1.856490979,	1.844087147,	1.801286156,	1.7376904,
               1.669804195,	1.607655387,	1.548374095,	1.4949701,
               1.447065626,	1.399607643,  1.354384982,	1.318725392,
               1.294773864,	1.279786607,	1.268049775,	1.257045807,
               1.248951736,	1.243210965,	1.239058923,	1.236026043,
               1.23309632,	1.229128787,	1.223387196,	1.215887987,
               1.207869122,	1.198749205,	1.186540058,	1.170622969,
               1.151783904)
labels <- as.character(Populacao)
labels <- gsub("." , "," , labels, fixed=T)
base_graf2<- data.frame(Ano,Populacao, labels)

## Gráfico 3
# Valores Hipoteticos
Pais <- factor(c("Ìndia","Japão","Alemanha","Reino Unido","França",
                 "Arábia Saudita","Rússia","China","Estados Unidos"))
Gasto <- c(40,50,50,60,70,40,90,207,640)
base_graf3<- data.frame(Pais,Gasto)
base_graf3$pos <- ifelse(base_graf3$Pais == 'Estados Unidos',2,1)

library(dplyr)
base_graf3 <- base_graf3 %>%
  group_by(pos) %>%
  mutate(total = sum(Gasto))

## Gráfico 4
Ano <- seq(2009,2020,1)
Exabytes <- c(130,360,750,1400,3100,6000,10000,14000,19000,25000,31500,40000)

base_graf4 <- data.frame(Ano,Exabytes)

## Gráfico 5

Ano <- seq(2004,2013,1)
Mundo <- c(4.2,3.8,4.1,4,1.5,-2,4,2.9,2.4,2.1)
UE <- c(2.6,2.2,3.4,3.2,0.6,-4.6,2.1,1.7,-0.4,0)

base_graf5 <- data.frame(Ano,Mundo,UE)
base_graf5<- melt(base_graf5, id=c("Ano"))

## Gráfico 6
pais <- c("Áustria","Bélgica","Bulgária","Croácia","Chipre","República Tcheca",
          "Dinamarca","Estônia","Finlândia","França","Alemanha","Grécia",
          "Hungria","Irlanda","Itália","Letônia","Lituânia","Luxemburgo",
          "Malta","Holanda","Polônia","Portugal","România",
          "Eslováquia","Eslovênia","Espanha","Suécia","Reino Unido")

petroleo <- c(90,100,97,72,104,96,-37,60,90,97,96,100,
              81,102,89,104,90,98,98,97,96,98,52,89,107,96,95,38)

base_graf6 <- data.frame(pais,petroleo)

## Gráfico 7
pais <- c("África do Sul","Brasil","Federação Russa",
          "China","Índia","OCDE")

final <- c(0.695,0.55,0.43,0.413,0.38,0.315)

inicio <- c(0.67,0.61,0.4,0.33,0.32,0.297)

base_graf7 <- data.frame(pais,final,inicio)

## Gráfico 8
# link <- 'https://github.com/ipeadata-lab/ipeaplot/files'
# base_graf8 <- data.table::fread(paste0(link,'/12239277/dados_grafico9.csv'),
#                                 dec = ",", encoding = 'Latin-1')

base_graf8 <- data.table(pais = c("Nigéria", "Vietnã", "Índia", "Indonésia", 
                         "Malásia", "China", "Arábia Saudita", "África do Sul", "México", 
                         "Turquia", "Brasil", "Argentina", "Austrália", "Rússia", "Polônia", 
                         "Canadá", "USA", "Fraça", "Reino Unido", "Espanha", "Coreia do Sul", 
                         "Itália", "Japão", "Alemanha"), 
                `crescimento medio do PIB per capita_perct` = c(2,0.5, 0.8, 0.5, 1, 0.1, 1.1, 0.4, 0.5, 0.5, 0.3, 0.5, 0.7, -0.2,
                                                                -0.1, 0.5, 0.5, 0.3, 0.32, 0.3, 0.12, 0.1, -0.1, 0),
                `crescimento populacional medio_perct` = c(3.9,5.3, 4.6, 4.2, 3.65, 4.2, 3.1, 3.62, 3.46, 3.3, 3.4, 3.1, 2.5, 
                                                           3.37, 3.1, 2.2, 2.2, 2.2, 2.18, 2.1, 2.28, 1.9, 2.1, 1.8),
                `crescimento do PIB (ppc)_perct` = c(5.9,5.8, 5.4, 4.7, 4.65, 4.3, 4.2, 4.02, 3.96, 3.8, 3.7, 3.6, 3.2, 
                                                     3.17, 3, 2.7, 2.7, 2.5, 2.5, 2.4, 2.4, 2, 2, 1.8))
```


## Gráfico 1


### Original
```{r, echo=FALSE, message=FALSE, warning=FALSE, fig.align="center", out.width = "100%"}
 knitr::include_graphics(paste0(getwd(),"/prints_original/grafico1.png"))
#![graph](prints_original/grafico1.png){fig-align="center", width='80%'}
```


### Padrão ipea
```{r eval=FALSE, message=FALSE, warning=FALSE}
graf1 <- ggplot(data=base_graf1, aes(x=as.numeric(as.character(Ano)), y=Populacao)) +
  geom_bar(stat="identity", fill = '#015f96', width=2) +
  geom_rug(aes(x = as.numeric(as.character(Ano)) - 2.5),
           outside = TRUE, sides = "b", length = unit(2, "mm"), linewidth = 0.25) +
  coord_cartesian(expand = TRUE, clip = "off") +
  labs(fill = "",
       title="GRÁFICO 1",
       subtitle="Evolução do crescimento da população mundial 2000-2050\n(Em bilhões)",
       caption = 'Fonte: United Nations (2012a).\nTradução dos autores.') +
  theme_ipea(x_breaks = 11,include_ticks = F,
             include_x_text_title = F,include_y_text_title = F) +
  geom_text(aes(label = gsub("\\.", ",",round(Populacao,1))),
            hjust = 0.5, vjust = -0.5)
```


```{r eval=FALSE, message=FALSE, warning=FALSE, results='hide'}
png(paste0(getwd(),"/prints_ipeaplot/grafico1.png"),  width=724, height=546)
print(graf1)
dev.off()

```

```{r, echo=FALSE, message=FALSE, warning=FALSE, fig.align="center", out.width = "100%"}
 knitr::include_graphics(paste0(getwd(),"/prints_ipeaplot/grafico1.png"))
```


## Gráfico 2 


### Original
```{r, echo=FALSE, message=FALSE, warning=FALSE, fig.align="center", fig.width=6, fig.height=6}
knitr::include_graphics(paste0(getwd(),"/prints_original/grafico2.png"))
```


### Padrão Ipea
```{r eval=FALSE, message=FALSE, warning=FALSE}
base_graf2$Ano <- as.numeric(as.character(base_graf2$Ano))
ref <- (subset(base_graf2,substr(Ano,4,4) %in% c('5','0')))

graf2 <- ggplot(base_graf2, aes(x=Ano, y=Populacao, group = 1))+
  geom_line(color = '#015f96') +
  labs(fill = "",
       title="GRÁFICO 2",
       subtitle="Taxa de crescimento da população mundial (Em %)",
       caption = 'Elaboração dos autores.') +
  theme_ipea(x_breaks = 14, x_text_angle = 90, include_ticks = F,expand_y_limit = T,
             include_x_text_title = F,include_y_text_title = F) +
  geom_rug(data = ref, aes(x = Ano+2),inherit.aes = FALSE,
           outside = TRUE, sides = "b", length = unit(2, "mm"), linewidth = 0.25) +
  coord_cartesian(clip = "off")
```


```{r eval=FALSE, message=FALSE, warning=FALSE, results='hide'}
png(paste0(getwd(),"/prints_ipeaplot/grafico2.png"),  width=724, height=546)
print(graf2)
dev.off()

```

```{r, echo=FALSE, message=FALSE, warning=FALSE, fig.align="center", out.width = "100%"}
 knitr::include_graphics(paste0(getwd(),"/prints_ipeaplot/grafico2.png"))

```


## Gráfico 3


### Original
```{r, echo=FALSE, message=FALSE, warning=FALSE, fig.align="center", fig.width=6, fig.height=6}
knitr::include_graphics(paste0(getwd(),"/prints_original/grafico3.png"))
```


### Padrão Ipea
```{r eval=FALSE, message=FALSE, warning=FALSE}
temp <- base_graf3 %>% distinct(pos, .keep_all = T)


graf3 <- ggplot(base_graf3, aes(x= as.character(pos), y=Gasto, fill=Pais,
                                label = Pais))+
  geom_bar(width = 0.5, stat = "identity", color= NA) +
  labs(y="Gastos com defesa",
       fill = "",
       title="GRÁFICO 3",
       subtitle="Gastos com defesa (Em bilhões de dólares)",
       caption = 'Fonte: Peter G. Peterson Foundation (2013).\nTradução dos autores.') +
  scale_fill_ipea() +
  geom_text( position = position_stack(vjust = 0.5), color = 'white') +
  geom_text(data = temp, aes(y = total ,label = paste0("U$ ",total)), vjust = -.5) +
  theme_ipea(legend.position="none",
             include_x_text_title = F) +
  theme(axis.ticks.x = ggplot2::element_blank(),
        axis.text.x  = ggplot2::element_blank())
```


```{r eval=FALSE, message=FALSE, warning=FALSE, results='hide'}
png(paste0(getwd(),"/prints_ipeaplot/grafico3.png"),  width=724, height=546)
print(graf3)
dev.off()

```

```{r, echo=FALSE, message=FALSE, warning=FALSE, fig.align="center", out.width = "100%"}
 knitr::include_graphics(paste0(getwd(),"/prints_ipeaplot/grafico3.png"))
```


## Gráfico 4


### Original
```{r, echo=FALSE, message=FALSE, warning=FALSE, fig.align="center", fig.width=6, fig.height=6}
knitr::include_graphics(paste0(getwd(),"/prints_original/grafico4.png"))
```


### Padrão Ipea
```{r eval=FALSE, message=FALSE}
graf4 <- ggplot(base_graf4, aes(x=Ano)) +
  geom_ribbon(aes(ymin=0, ymax=Exabytes), fill = "#015f96")+
  labs(fill = "",
       title="GRÁFICO 4",
       subtitle="Evolução do volume de informação digital até 2020",
       caption = 'Fonte: IDC’s Digital Universe Study (Gantz e Reinsel, 2012).') +
  theme_ipea(legend.position="none",expand_x_limit = F,
             x_breaks = 10, y_breaks = 4,
             include_x_text_title = F,include_y_text_title = F)
```


```{r eval=FALSE, message=FALSE, warning=FALSE, results='hide'}
png(paste0(getwd(),"/prints_ipeaplot/grafico4.png"),  width=724, height=546)
print(graf4)
dev.off()


```

```{r, echo=FALSE, message=FALSE, warning=FALSE, fig.align="center", out.width = "100%"}
 knitr::include_graphics(paste0(getwd(),"/prints_ipeaplot/grafico4.png"))
```


## Gráfico 5


### Original
```{r, echo=FALSE, message=FALSE, warning=FALSE, fig.align="center", fig.width=6, fig.height=6}
knitr::include_graphics(paste0(getwd(),"/prints_original/grafico5.png"))
```


### Padrão Ipea
```{r eval=FALSE, message=FALSE, warning=FALSE}
graf5 <- ggplot(base_graf5, aes(Ano, value, group=variable, color=variable))+
  geom_line(linewidth=1)+
  geom_point(aes(shape=variable, fill = variable),
             size=3, stroke=1.5, show.legend = F)+
  labs(color = "",
       title="GRÁFICO 5",
       subtitle="Crescimento anual do PIB da União Europeia (2004-2013) (Em %)",
       caption = 'Fonte: World Bank (2014).\nTradução dos autores.\nObs.: Os dados foram coletados no sítio do Banco Mundial.\nDisponível em: <http://goo.gl/IvtZ1a>. Acesso em: 26 set. 2014.') +
  theme_ipea(x_breaks = 10, y_breaks = 10, legend.position = 'bottom',
             expand_y_limit = T,expand_x_limit = F,
             include_x_text_title = F,include_y_text_title = F) +
  scale_color_ipea(palette_direction = -1, ) +
  scale_shape_manual(values = c(19, 21)) +
  scale_fill_manual(values = c(NA,"white"))
```


```{r eval=FALSE, message=FALSE, warning=FALSE, results='hide'}
png(paste0(getwd(),"/prints_ipeaplot/grafico5.png"),  width=724, height=546)
print(graf5)
dev.off()

```

```{r, echo=FALSE, message=FALSE, warning=FALSE, fig.align="center", out.width = "100%"}
 knitr::include_graphics(paste0(getwd(),"/prints_ipeaplot/grafico5.png"))
```


## Gráfico 6


### Original
```{r, echo=FALSE, message=FALSE, warning=FALSE, fig.align="center", fig.width=6, fig.height=6}
knitr::include_graphics(paste0(getwd(),"/prints_original/grafico6.png"))
```


### Padrão Ipea
```{r eval=FALSE, message=FALSE, warning=FALSE}
base_graf6$pais<- factor(base_graf6$pais, levels=rev(sort(base_graf6$pais)))
graf6 <- ggplot(data=base_graf6, aes(x=pais, y=petroleo)) +
  geom_bar(stat="identity", fill = '#015f96') +
  coord_flip() +
  labs(fill = "",
       title="GRÁFICO 6",
       subtitle="Importações de energia em termos do consumo total\nde energia dos países da União Europeia",
       caption = 'Fonte: Eurostat (2014).\nTradução dos autores.') +
  theme_ipea(legend.position = 'none', expand_x_limit = F,
             include_x_text_title = F,include_y_text_title = F)
```


```{r eval=FALSE, message=FALSE, warning=FALSE, results='hide'}
png(paste0(getwd(),"/prints_ipeaplot/grafico6.png"),  width=724, height=546)
print(graf6)
dev.off()


```

```{r, echo=FALSE, message=FALSE, warning=FALSE, fig.align="center", out.width = "100%"}
 knitr::include_graphics(paste0(getwd(),"/prints_ipeaplot/grafico6.png"))
```


## Gráfico 7


### Original
```{r, echo=FALSE, message=FALSE, warning=FALSE, fig.align="center", fig.width=6, fig.height=6}
knitr::include_graphics(paste0(getwd(),"/prints_original/grafico7.png"))
```


### Padrão Ipea
```{r eval=FALSE, message=FALSE, warning=FALSE}
base_graf7<- melt(base_graf7, id=c("pais"))

graf7 <- ggplot(data=base_graf7, aes(x=pais, y=value, group=variable, fill=variable)) +
  geom_bar(position='dodge',stat="identity") +
  coord_flip()+
  labs(fill = "",
       title="GRÁFICO 7",
       subtitle="Mudanças nos níveis de desigualdade do BRICS, calculadas pelo coeficiente\nGini, entre o início da década de 1990 e o final da década de 2000",
       caption = '') +
  theme_ipea(legend.position = 'bottom',
             include_x_text_title = F,include_y_text_title = F) +
  scale_fill_ipea()
```


```{r eval=FALSE, message=FALSE, warning=FALSE, results='hide'}
png(paste0(getwd(),"/prints_ipeaplot/grafico7.png"),  width=724, height=546)
print(graf7)
dev.off()

```

```{r, echo=FALSE, message=FALSE, warning=FALSE, fig.align="center", out.width = "100%"}
 knitr::include_graphics(paste0(getwd(),"/prints_ipeaplot/grafico7.png"))
```


## Gráfico 8


### Original
```{r, echo=FALSE, message=FALSE, warning=FALSE, fig.align="center", fig.width=6, fig.height=6}
knitr::include_graphics(paste0(getwd(),"/prints_original/grafico8.png"))
```


### Padrão Ipea
```{r eval=FALSE, message=FALSE, warning=FALSE}
######## GRAFICO 8
base_graf8_aux <- base_graf8[,c(1,4)]
base_graf8_aux$n <- 1:nrow(base_graf8_aux)
base_graf8_new <- base_graf8[,-4]
base_graf8_new<- melt(base_graf8_new, id=c("pais"))

base_graf8_new <- base_graf8_new %>%
  group_by(variable) %>%
  mutate(n = 1:n())

graf8 <-
ggplot() +
  geom_bar(data=base_graf8_new, aes(x = reorder(pais, n), y=value,
                                    group=variable, fill=variable),
           stat="identity", width = 0.5) +
  labs(fill = "",
       title="GRÁFICO 8",
       subtitle="Crescimento médio real do PIB em PPC (2011-2050) (Em %)",
       caption = 'Fonte: PwC (2014).\nTradução dos autores') +
  geom_line(data=base_graf8_aux, aes(x = reorder(pais, n) ,y=`crescimento do PIB (ppc)_perct`, group = 1),
            color = 'gray') +
  scale_fill_ipea() +
  theme_ipea(legend.position = 'bottom', x_text_angle = 90) +
  coord_cartesian(clip = "off")
```


```{r eval=FALSE, message=FALSE, warning=FALSE, results='hide'}
png(paste0(getwd(),"/prints_ipeaplot/grafico8.png"),  width=724, height=546)
print(graf8)
dev.off()

```


```{r, echo=FALSE, message=FALSE, warning=FALSE, fig.align="center", out.width = "100%"}
 knitr::include_graphics(paste0(getwd(),"/prints_ipeaplot/grafico8.png"))
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ipea_label_size.R
\name{ipea_label_size}
\alias{ipea_label_size}
\title{Font size for data labels in Ipea graphics}
\usage{
ipea_label_size(pt = 6)
}
\arguments{
\item{pt}{Numeric. Desired font size in points. Defaults to \code{6}, the size
used by the editorial guidelines of the Institute for Applied
Economic Research (Ipea) for labels drawn over data (e.g. values on
bars or lines). See
\url{https://www.ipea.gov.br/manualeditorial/padroes/padroes-grafico-visuais/ilustracoes-pi.html}.}
}
\value{
A numeric value (in mm) to be passed to the \code{size}
argument/aesthetic of \code{ggplot2::geom_text()}, \code{ggplot2::geom_label()}
or similar geoms.
}
\description{
Converts a font size in points (pt) into the \code{size}
argument/aesthetic (in mm) expected by \code{ggplot2::geom_text()},
\code{ggplot2::geom_label()} and similar geoms. \code{ggplot2} specifies text size
for geoms in mm and internally converts it to points by multiplying by
\code{2.845276} (\code{72.27/25.4}); this helper performs the inverse conversion so
that labels are rendered at an exact point size.
}
\examples{
library(ggplot2)
ggplot(mtcars[1:5, ], aes(hp, mpg, label = round(mpg))) +
  geom_text(size = ipea_label_size(), colour = "black") +
  theme_ipea()
}
\seealso{
Other ggplot2 theme functions:
\code{\link[=ipea_contrast_color]{ipea_contrast_color()}},
\code{\link[=scale_color_ipea]{scale_color_ipea()}},
\code{\link[=scale_fill_ipea]{scale_fill_ipea()}},
\code{\link[=theme_ipea]{theme_ipea()}}
}
\concept{ggplot2 theme functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ipea_palette.R
\name{ipea_palette}
\alias{ipea_palette}
\title{Ipea Color Palette and Scales}
\usage{
ipea_palette(
  palette = c("Blue", "Green", "Orange", "Pink", "Pink-Deep", "Green-Blue",
    "Green-Blue-White", "Red-Blue", "Red-Blue-White", "Orange-Blue", "Orange-Blue-White",
    "Viridis", "Inferno", "Magma", "Plasma", "Cividis"),
  n,
  alpha = 1,
  begin = 0,
  end = 1,
  palette_direction = 1
)
}
\arguments{
\item{palette}{A character string indicating the color map option to use.
These options are available:'Blue', 'Green', 'Orange', 'Pink', 'Pink-Deep', 'Red-Blue'
'Orange-Blue', 'Green-Blue', 'Red-Blue-White', 'Orange-Blue-White',
'Green-Blue-White', 'Viridis', 'Inferno', 'Magma', 'Plasma', 'Cividis'.}

\item{n}{The number of colors (\eqn{\ge 1}) used in the palette.}

\item{alpha}{The alpha transparency in a number between \code{0} and \code{1}.}

\item{begin}{The (corrected) hue in a number between \code{0} and \code{1} at which the
color map begins.}

\item{end}{The (corrected) hue in a number between \code{0} and \code{1} at which the
color map ends.}

\item{palette_direction}{Sets the order of colors in the scale. If \code{1}, the default,
colors are ordered from darkest to lightest. If \code{-1}, the order of
colors is reversed.}
}
\value{
\code{ipea_palette} produces a character vector, \code{cv}, containing color hex codes.
This vector can be utilized to establish a custom color scheme for future graphics using \code{palette(cv)},
or it can be applied directly as a \code{col =} parameter in graphic functions or within \code{par}.
}
\description{
This function creates a vector of n equally spaced colors
along the selected color map.
}
\details{
A 9-color Ipea palette.
}
\references{
'Blue','Green','Orange','Pink','Pink-Deep','Green-Blue','Green-Blue-White','Red-Blue','Red-Blue-White',
'Orange-Blue','Orange-Blue-White', 'Viridis','Inferno',
'Magma','Plasma' and 'Cividis': https://pmassicotte.github.io/paletteer_gallery/
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ipeaplot.R
\docType{package}
\name{ipeaplot}
\alias{ipeaplot}
\alias{ipeaplot-package}
\title{ipeaplot: ggplot Graphics in Ipea Standard}
\description{
Convenient functions to create ggplot graphics following the editorial
guidelines of the  Institute for Applied Economic Research - Ipea.
}
\section{Usage}{

Please check the vignettes and data documentation on the
\href{https://github.com/ipeadata-lab/ipeaplot}{website}.
}

\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/ipeadata-lab/ipeaplot}
  \item Report bugs at \url{https://github.com/ipeadata-lab/ipeaplot/issues}
}

}
\author{
\strong{Maintainer}: Pedro Ferreira \email{pedro.ferreira2@ipea.gov.br}

Authors:
\itemize{
  \item Pedro Ferreira \email{pedro.ferreira2@ipea.gov.br}
  \item Pedro Jorge \email{pedro.alves@ipea.gov.br}
  \item Daniel Lima \email{daniel.viegas@ipea.gov.br}
  \item Gustavo Coelho \email{gustavo.coelho@ipea.gov.br}
  \item Rafael H. M. Pereira \email{rafa.pereira.br@gmail.com}
  \item Lucas Mation \email{lucas.mation@ipea.gov.br}
}

Other contributors:
\itemize{
  \item Fabio Vaz \email{fabio.vaz@ipea.gov.br} [contributor]
  \item Ipea - Institue for Applied Economic Research [copyright holder, funder]
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scale_ipea.R
\name{scale_fill_ipea}
\alias{scale_fill_ipea}
\title{Scale fill IPEA}
\usage{
scale_fill_ipea(
  palette = c("Blue", "Green", "Orange", "Pink", "Pink-Deep", "Green-Blue",
    "Green-Blue-White", "Red-Blue", "Red-Blue-White", "Orange-Blue", "Orange-Blue-White",
    "Viridis", "Inferno", "Magma", "Plasma", "Cividis", "NT", "NT-Categorical", "TD",
    "TD-Categorical"),
  palette_direction = 1,
  decimal.mark = ",",
  barheight = NULL,
  barwidth = NULL,
  title.hjust = NULL,
  label.hjust = NULL,
  ...
)
}
\arguments{
\item{palette}{A character vector specifying the available palette for the
color palette. The default palette are \code{"Blue"}, but we can also change
to \code{'Green'}, \code{'Orange'}, \code{'Pink'}, \code{'Pink-Deep'}, \code{'Red-Blue'}, \code{'Orange-Blue'}, \code{'Green-Blue'},
\code{'Red-Blue-White'}, \code{'Orange-Blue-White'}, \code{'Green-Blue-White'},
\code{'Viridis'}, \code{'Inferno'}, \code{'Magma'}, \code{'Plasma'}, \code{'Cividis'}, \code{'NT'}, \code{'TD'}.
\code{'NT'} and \code{'TD'} automatically switch from their sequential (continuous)
colours to a 12-colour qualitative set when the mapped variable is
discrete -- there is no need to request \code{'NT-Categorical'}/\code{'TD-Categorical'}
directly, though those names remain available for \code{ipea_pal()}/\code{ipea_palette()}.}

\item{palette_direction}{A logical argument specifying if the ordering of the colors
will follow the default of the palette (when the argument is 1) or if it will
have an inverted ordering (for cases where it is 0).}

\item{decimal.mark}{The character to be used to indicate the numeric decimal point and
Character used between every 3 digits to separate thousands.
By default, the function uses a comma \code{","}, following
the format used in Brazilian Portuguese.}

\item{barheight}{The height of the color gradient bar. This parameter is used
when the direction is set to "horizontal".}

\item{barwidth}{The width of the color gradient bar. This parameter is used
when the direction is set to "horizontal".}

\item{title.hjust}{A number specifying horizontal justification of the title text.}

\item{label.hjust}{A number specifying vertical justification of the title text.}

\item{...}{Additional arguments to be passed to the scale_fill_gradientn,
scale_color_gradientn, scale_fill_distiller or scale_color_distiller
function from the ggplot2 package}
}
\value{
A list object be added to a ggplot object to change color pallete.
}
\description{
Generate a fill palette (continuous or discrete) in the
formatting of texts published by the Institute for Applied
Economic Research (IPEA)
}
\examples{
# Creating scale for ggplot2 graph using default arguments
library(ggplot2)
fig_raw <- ggplot() +
  geom_col(data = mtcars, aes(x = hp , y = mpg, fill = cyl)) +
  scale_fill_ipea()

# Creating scale for ggplot2 graph using green sequential palette
fig_raw <- ggplot() +
  geom_col(data = mtcars, aes(x = hp , y = mpg, fill = cyl)) +
  scale_fill_ipea(palette = "Green")

# "TD" uses the sequential blue ramp for a numeric variable, and the
# 12-colour qualitative set for a discrete one, automatically
fig_raw <- ggplot() +
  geom_col(data = mtcars, aes(x = hp , y = mpg, fill = factor(cyl))) +
  scale_fill_ipea(palette = "TD")
}
\seealso{
Other ggplot2 theme functions:
\code{\link[=ipea_contrast_color]{ipea_contrast_color()}},
\code{\link[=ipea_label_size]{ipea_label_size()}},
\code{\link[=scale_color_ipea]{scale_color_ipea()}},
\code{\link[=theme_ipea]{theme_ipea()}}
}
\concept{ggplot2 theme functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/save_ipeaplot.R
\name{save_ipeaplot}
\alias{save_ipeaplot}
\title{Save ggplot in multiple formats}
\usage{
save_ipeaplot(
  gplot = ggplot2::last_plot(),
  file.name,
  format = NULL,
  width = 6.3,
  height = 3.94,
  units = c("in", "mm", "cm", "px"),
  dpi = 300,
  background = "white",
  quality = 95,
  path = ".",
  scale = 1,
  overwrite = TRUE,
  include_date = FALSE,
  date_format = "\%Y\%m\%d",
  use_cairo = TRUE,
  use_ragg = TRUE,
  quiet = TRUE,
  ...
)
}
\arguments{
\item{gplot}{ggplot object. Default: ggplot2::last_plot().}

\item{file.name}{Base name of the file (without extension). E.g., "results/figs/my_plot".}

\item{format}{Vector with one or more of: "eps", "jpg", "pdf", "png", "svg".
If NULL, uses getOption("ipea.plot.default_format", "eps").}

\item{width, height}{Plot dimensions (default 160 x 100).}

\item{units}{Units: "mm", "cm", "in", "px" (default "in").}

\item{dpi}{DPI for raster formats (default 300).}

\item{background}{Background color; use NA for transparent PNGs.
For JPEG (no transparency), NA becomes "white".}

\item{quality}{JPEG quality (0 to 100) (default 95).}

\item{path}{Output directory (default ".").}

\item{scale}{Scale factor passed to ggsave (default 1).}

\item{overwrite}{If FALSE, does not overwrite existing files: creates suffixes _1, _2, ... (default TRUE).}

\item{include_date}{If TRUE, appends the date to the file name: _YYYYMMDD (default FALSE).}

\item{date_format}{Date format (default "\%Y\%m\%d").}

\item{use_cairo}{Prefer using Cairo for PDF/EPS (default TRUE).}

\item{use_ragg}{Prefer using ragg for PNG/JPG when available (default TRUE).}

\item{quiet}{Suppress messages from ggsave when supported (default TRUE).}

\item{...}{Passed to ggplot2::ggsave() (and therefore to the underlying graphics devices when applicable).}
}
\value{
Named (invisible) vector with the paths of the saved files.
}
\description{
Unified function to save ggplot objects in one or more formats.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme_ipea.R
\name{theme_ipea}
\alias{theme_ipea}
\title{Ggplot theme for Ipea charts and figures}
\usage{
theme_ipea(
  axis_lines = "full",
  axis_values = TRUE,
  legend.position = "bottom",
  grid.adjust = "horizontal",
  x_breaks = NULL,
  y_breaks = NULL,
  expand_x_limit = TRUE,
  expand_y_limit = TRUE,
  x_text_angle = 0,
  include_x_text_title = TRUE,
  include_y_text_title = TRUE,
  include_ticks = TRUE,
  ...
)
}
\arguments{
\item{axis_lines}{A character vector specifying the axis style. Valid options are
\code{"none"} (no axis lines), \code{"full"} (full-length axis lines), and
\code{"half"} (half-length axis lines), the default.}

\item{axis_values}{Logical value indicating whether to show text elements. If \code{TRUE},
axis text will be displayed in black; otherwise, they will
be hidden.}

\item{legend.position}{A character vector specifying the position of the
legend. Valid options are \code{"bottom"} (default, matching the Ipea
editorial guidelines), \code{"right"}, \code{"left"}, and \code{"top"}.}

\item{grid.adjust}{Defines whether the grid lines should be \code{"horizontal"}
(default) or \code{"vertical"}.}

\item{x_breaks}{Numeric. The number of breaks on the x-axis}

\item{y_breaks}{Numeric. The number of breaks on the y-axis}

\item{expand_x_limit}{Logical value that indicates whether the x-axis
boundary should be expanded. If \code{TRUE}, the x-axis limits will be
expanded; otherwise there will be no change}

\item{expand_y_limit}{Logical value that indicates whether the y-axis
boundary should be expanded. If \code{TRUE}, the x-axis limits will be
expanded; otherwise there will be no change}

\item{x_text_angle}{Numeric. Angle in degrees of the text in the x-axis.}

\item{include_x_text_title}{Logical. Whether to include x text title Defaults to \code{TRUE}.}

\item{include_y_text_title}{Logical. Whether to include x text title. Defaults to \code{TRUE}.}

\item{include_ticks}{Logical. Whether to include ticks. Defaults to \code{TRUE}.}

\item{...}{Additional arguments to be passed to the \code{theme} function from the
\code{ggplot2} package.}
}
\value{
A custom theme for IPEA graphics.
}
\description{
Applies a custom theme for ggplot figures following the editorial
guidelines used by the Institute for Applied Economic Research - Ipea. The
function includes standardized formatting of options for axis lines, text,
}
\examples{
# Creating theme for ggplot2 graph using default arguments
library(ggplot2)
fig_raw <- ggplot() +
  geom_col(data = mtcars, aes(x = hp , y = mpg, fill = cyl)) +
  theme_ipea()

}
\seealso{
Other ggplot2 theme functions:
\code{\link[=ipea_contrast_color]{ipea_contrast_color()}},
\code{\link[=ipea_label_size]{ipea_label_size()}},
\code{\link[=scale_color_ipea]{scale_color_ipea()}},
\code{\link[=scale_fill_ipea]{scale_fill_ipea()}}
}
\concept{ggplot2 theme functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ipea_contrast_color.R
\name{ipea_contrast_color}
\alias{ipea_contrast_color}
\title{Contrasting text colour for labels drawn over a fill colour}
\usage{
ipea_contrast_color(background)
}
\arguments{
\item{background}{A character vector of colours (hex codes or R colour
names) used as the fill/background behind the label.}
}
\value{
A character vector the same length as \code{background}, containing
\code{"white"} or \code{"black"}.
}
\description{
Given one or more fill/background colours, returns \code{"white"}
or \code{"black"}, whichever gives better contrast against that background.
Useful for value labels drawn inside bars or over points/lines, matching
the Ipea editorial guidelines for graphics, which alternate the colour of
data labels according to how dark the colour behind them is. See
\url{https://www.ipea.gov.br/manualeditorial/padroes/padroes-grafico-visuais/ilustracoes-pi.html}.
}
\examples{
ipea_contrast_color(c("#29112F", "#D2A0C9"))

library(ggplot2)
df <- data.frame(grupo = c("A", "B"), valor = c(22.6, 4.7),
                  cor = c("#29112F", "#D2A0C9"))
ggplot(df, aes(grupo, valor, fill = cor, label = valor)) +
  geom_col() +
  geom_text(aes(color = ipea_contrast_color(cor)), size = ipea_label_size(),
            fontface = "bold") +
  scale_fill_identity() +
  scale_color_identity() +
  theme_ipea()
}
\seealso{
Other ggplot2 theme functions:
\code{\link[=ipea_label_size]{ipea_label_size()}},
\code{\link[=scale_color_ipea]{scale_color_ipea()}},
\code{\link[=scale_fill_ipea]{scale_fill_ipea()}},
\code{\link[=theme_ipea]{theme_ipea()}}
}
\concept{ggplot2 theme functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ipea_palette.R
\name{ipea_pal}
\alias{ipea_pal}
\title{Ipea palette}
\usage{
ipea_pal(
  palette = c("Blue", "Green", "Orange", "Pink", "Pink-Deep", "Green-Blue",
    "Green-Blue-White", "Red-Blue", "Red-Blue-White", "Orange-Blue", "Orange-Blue-White",
    "Viridis", "Inferno", "Magma", "Plasma", "Cividis", "NT", "NT-Categorical", "TD",
    "TD-Categorical"),
  alpha = 1,
  begin = 0,
  end = 1,
  palette_direction = 1
)
}
\arguments{
\item{palette}{A character string indicating the color map option to use.
These options are available: 'Blue', 'Green', 'Orange', 'Pink', 'Pink-Deep', 'Red-Blue'
'Orange-Blue', 'Green-Blue', 'Viridis', 'Inferno', 'Magma', 'Plasma'
'Cividis', 'NT', 'NT-Categorical', 'TD', 'TD-Categorical'.}

\item{alpha}{The alpha transparency in a number between \code{0} and \code{1}.}

\item{begin}{The (corrected) hue in a number between \code{0} and \code{1} at which the color map begins.}

\item{end}{The (corrected) hue in a number between \code{0} and \code{1} at which the color map ends.}

\item{palette_direction}{Sets the order of colors in the scale. If 1, the default,
colors are ordered from darkest to lightest. If -1, the order of
colors is reversed.}
}
\value{
\code{ipea_palette} produces a character vector, \code{cv}, containing color hex codes.
This vector can be utilized to establish a custom color scheme for future graphics using \code{palette(cv)},
or it can be applied directly as a \code{col =} parameter in graphic functions or within \code{par}.
}
\description{
Ipea palette
}
\examples{
scales::show_col(ipea_pal()(10))
scales::show_col(ipea_pal(palette_direction = -1)(6))
scales::show_col(ipea_pal(begin = 0.2, end = 0.8)(4))
scales::show_col(ipea_pal(palette = "Green")(6))
scales::show_col(ipea_pal(palette = "NT")(10))
scales::show_col(ipea_pal(palette = "TD-Categorical")(12))
}
\references{
'Blue','Green','Orange','Pink','Pink-Deep','Green-Blue','Green-Blue-White','Red-Blue','Red-Blue-White',
'Orange-Blue','Orange-Blue-White', 'Viridis','Inferno',
'Magma','Plasma' and 'Cividis': https://pmassicotte.github.io/paletteer_gallery/
'NT'/'TD' and their '-Categorical' variants: see \code{ipea_palette()}.
}
